source [file join $xmllab bin tcl env.tcl]

set xmllab_escaped [string map {"\\" "/"} $xmllab]

set xsltproc [file join $xmllab_bin xsltproc]

if {$tcl_platform(os)=="Linux" || $tcl_platform(os)=="Darwin"} {
	set ld LD_LIBRARY_PATH
	if {$tcl_platform(os)=="Darwin"} {
		set ld DYLD_LIBRARY_PATH
	}

	puts "Building the XMLlab compile script"
	set f [open [file join $xmllab_bin xmllab] w]
	puts $f "export xmllab=\"$xmllab\""
	puts $f "export lang=$lang"
	puts $f "tclsh  \"$xmllab\"/bin/tcl/xmllab.tcl \"\$\@\""
	close $f
	exec chmod a+x [file join $xmllab_bin xmllab]

	puts "Building the XML catalog"

	catch {exec env $ld=$xmllab_lib \
                        $xsltproc --stringparam xmllab $xmllab\
                        --output dtd/catalog.xml\
			dtd/catalog.xsl dtd/catalog.xml.in} mess

    # installation of catalog.xml in xxe (>=2.11) config dir

    set catalogdir [file join $env(HOME) .xxe addon config] 
    file mkdir $catalogdir
    file copy -force dtd/catalog.xml $catalogdir

	set env(XML_CATALOG_FILES) "file:///[string map {" " "%20" "\\" "/"} $xmllab]dtd/catalog.xml"

	catch {exec env $ld=$xmllab_lib\
                        XML_CATALOG_FILES=$env(XML_CATALOG_FILES)\
		    $xsltproc --stringparam xmllab "$xmllab_escaped"\
		    --stringparam lang $lang\
		    --output tmp.xml\
		    $xmllab/xsl/xmllab_menu.xsl\
		    $xmllab/examples/demos.xml} mess

	catch {exec env $ld=$xmllab_lib\
	            $xsltproc --output $xmllab/menu.sce\
		    $xmllab/xsl/sciml.xsl\
		    tmp.xml} mess

} elseif {$tcl_platform(os)=="Windows NT"} {
	puts "Building the XMLlab compile script"
	set f [open [file join $xmllab_bin xmllab.bat] w]
	puts $f "@echo off"
	puts $f "set xmllab=$xmllab"
	puts $f "set lang=$lang"
	puts $f "set SCILAB=$SCI"
	puts $f "tclsh  \"%xmllab%bin\\tcl\\xmllab.tcl\" %1 %2 %3 %4 %5 %6 %7 %8"
	close $f

	puts "Building the XML catalog"
	set env(path) $xmllab_lib

	catch {exec $xsltproc --stringparam xmllab "/$xmllab"\
					--output dtd/catalog.xml\
					dtd/catalog.xsl dtd/catalog.xml.in} mess

    # installation of catalog.xml in xxe (>= 2.11) config dir

	set catalogdir [file join $env(HOME) "Application Data" XMLmind XMLeditor addon config]
    file mkdir $catalogdir
    file copy -force dtd/catalog.xml $catalogdir

	set env(XML_CATALOG_FILES) "file:///[string map {" " "%20" "\\" "/"} $xmllab]dtd/catalog.xml"

	catch {exec $xsltproc  --stringparam xmllab "$xmllab_escaped"\
		    --stringparam lang $lang\
		    --output tmp.xml\
		    $xmllab/xsl/xmllab_menu.xsl\
		    $xmllab/examples/demos.xml} mess

	catch {exec $xsltproc --output $xmllab/menu.sce\
		    $xmllab/xsl/sciml.xsl\
		    tmp.xml} mess
}

file delete tmp.xml

# Installation of the loading part in the user's startup file (.scilab)

insert  [file join $SCIHOME .scilab] "*XMLlab/loader.sce*" "exec(\"[file join $xmllab loader.sce]\",-1) // Load XMLlab"
