proc xmllab {argc argv} {

global env tcl_platform xmllab_bin xmllab lang TMPDIR

if !$argc {mess "usage : xmllab file or xmllab file.xml"; return 1}

if {[catch {set xmllab}]} {
	set xmllab $env(xmllab)
}

set errorCode NONE

set run 0
set packsource  no
set havesource  no
set screenshots no
set valid 0
set k 0
set job ""
set amp &
set appname NONAME
set lang DEFAULT

while {$k<$argc} {
	set arg [lindex $argv $k] 
	if {$arg=="-lang"} {
        incr k
        set lang [lindex $argv $k]
	} elseif {$arg=="-appname"} {
        incr k
        set appname [lindex $argv $k]    
    } elseif {$arg=="-pack"} {
    	set packsource yes
     } elseif {$arg=="-run"} {
	    set havesource yes
    	set run 1
    } elseif {$arg=="-screenshots"} {
    	set run 1
        set screenshots yes
    } elseif {$arg=="-valid"} {
    	set valid 1
    } else {
        set job $arg	
    }
    incr k
}

if {$job==""} {
	mess "xmllab : missing argument"; 
	return 1
}

set rootname [file rootname $job]
set filename [file tail $rootname]
set dirname [file dirname $rootname]

if {[file pathtype $dirname]=="relative"} {
	if {$dirname=="."} {
		set dirname ""
	}
	set dirname [file join [pwd] $dirname]
}

set ext [file extension $job]

if ![file exists $job] {
	if {$ext==""} {set ext .xml}
	set source $rootname$ext
	if ![file exists $source] {mess "$job, $rootname$ext : file not found"; return 1}
} else {
	set source $job
}

if $run {
    if {$screenshots=="no"} {
	    set filename [clock seconds]
    }
    set outputname [file join $TMPDIR $filename]
    set outputdir $TMPDIR
} else {
    if {[file writable [pwd]]} {
	set outputname [file join [pwd] $filename]
        set outputdir [pwd]
    } else {
	mess "You don't have write access in current directory"
        return 1
    }
}
set xsltproc [file join $xmllab_bin xsltproc]
set xmllint [file join $xmllab_bin xmllint]


set env(XML_CATALOG_FILES) "file:///[string map {" " "%20" "\\" "/"} $xmllab]dtd/catalog.xml"

set fileversion [exec $xsltproc $xmllab/xsl/fileversion.xsl $source]

if {$fileversion=="1.2"} {
    if {[catch {exec $xsltproc --novalid --stringparam file $filename $xmllab/xsl/convert1.3.xsl $source > $outputname-converted.xml} mess]} {
	    mess $mess
	    if {$errorCode!="NONE"} {
		    return 1;
	    }
    }
	file copy -force $rootname.xml $outputname.xml.1.2
	file rename -force $outputname-converted.xml $outputname.xml	
    set fileversion "1.3"
    set source $outputname.xml
}

if {$fileversion == "1.3"} {
    if {[catch {exec $xsltproc --novalid --stringparam file $filename $xmllab/xsl/convert1.4.xsl $source > $outputname-converted.xml} mess]} {
	    mess $mess
	    if {$errorCode!="NONE"} {
		    return 1;
	    }
    }
	file copy -force $rootname.xml  $outputname.xml.1.3
	file rename -force  $outputname-converted.xml  $outputname.xml	
    set fileversion "1.4"
    set source $outputname.xml
}

# Validation

if {[catch {exec $xmllint -noout --valid $source} mess]} {
	mess $mess
	if {$errorCode!="NONE"} {
		return 1;
	}
}

if $valid { # Validate only
	return 0
}

if {[catch {exec $xsltproc  --stringparam screenshots $screenshots\
                       --stringparam language $lang\
                      --stringparam os $tcl_platform(os)\
					   --stringparam file $filename\
					   --stringparam rootname $rootname\
			 	       --stringparam packsource $packsource\
					    $xmllab/xsl/simulation.xsl\
						 $source > $outputname.sciml} mess]} {
	mess $mess
	if {$errorCode!="NONE"} {
		return 1;
	}
}


if {[catch {exec  $xsltproc $xmllab/xsl/sciml.xsl $outputname.sciml > $outputname.sce} mess]} {
	mess $mess
	if {$errorCode!="NONE"} {
		return 1;
	}
}

if {$packsource=="no"} {
    if {[catch {exec $xsltproc --stringparam appname $appname\
               --stringparam screenshots $screenshots\
               --stringparam language $lang\
               --stringparam os $tcl_platform(os)\
	           --stringparam file $filename\
	           --stringparam dirname $dirname\
	           --stringparam sourcename $source\
	           --stringparam outputdir $TMPDIR\
                   --stringparam havesource $havesource\
	            $xmllab/xsl/simultk.xsl $source > $outputname.tksci} mess]} {
	    mess $mess
	    if {$errorCode!="NONE"} {
		    return 1;
	    }
    }       
	exec $xsltproc --stringparam file $filename --stringparam dirname $dirname $xmllab/xsl/tk.xsl $outputname.tksci > $outputname.tk
}

# images to be copied side by side with the sce and tk files

set images [exec $xsltproc $xmllab/xsl/images.xsl $source]

if {[string length $images]>0} {
	foreach file $images {
		catch {file copy -force [file join $dirname $file] [file join $outputdir $file]}
	}
}

if {[file exists [file join $dirname $filename.sci] ]} {	
	file copy -force [file join $dirname $filename.sci] [file join $outputdir $filename.sci]
}

if {[file exists [file join $dirname $filename.mod] ]} {
	file copy -force [file join $dirname $filename.mod] [file join $outputdir $filename.mod]
}

file delete $outputname.sciml
#file delete $outputname.tksci

if $argc==1 {
	return 0
}

if $run {
	if {[catch {ScilabEval 0}]} { # If we are outside Scilab
	    switch $tcl_platform(os) {
		    "Windows NT" {catch {exec cmd /c start /min $env(SCILAB)\\bin\\scilex -nwni -nb -f $outputname.sce &} mess}
		    "Linux" {catch {exec scilab -nwni -nb -f $outputname.sce > /dev/null &} mess}
		    "SunOs" {catch {exec scilab -nwni -nb -f $outputname.sce > /dev/null &} mess}
		    "Darwin" {catch {exec scilab -nwni -nb -f $outputname.sce >  /dev/null &} mess}
	    }
    } else { # If we were called from Scilab
		ScilabEval exec(\"$outputname.sce\",-1)
    }
}
return 0
}

proc publishlist {job htmlonly} {

    global env tcl_platform xmllab_bin xmllab lang TMPDIR

    set xsltproc [file join $xmllab_bin xsltproc]

    set rootname [file rootname $job]
    set filename [file tail $rootname]
    set dirname [file dirname $rootname]

    if {[file pathtype $dirname]=="relative"} {
	    if {$dirname=="."} {
		    set dirname ""
	    }
	    set dirname [file join [pwd] $dirname]
    }

    set ext [file extension $job]

    if ![file exists $job] {
	    if {$ext==""} {set ext .xml}
	    set source $rootname$ext
	    if ![file exists $source] {mess "$job, $rootname$ext : file not found"; return 1}
    } else {
	    set source $job
    }
    
    set source [file join $dirname $source]

	file mkdir $filename
    
    set oldpwd [pwd]
    cd $filename
	set imagesdir [file join $xmllab noarch images]
        
    foreach simulation [exec $xsltproc [file join $xmllab xsl publishlist.xsl] $source] {
    	set item  [file join $dirname $simulation]
        set root [file rootname [file tail $simulation]]
        set todo 0
        if ![file exists $root] {
        	set todo 1
        } elseif [file exists $root/sizes.xml] {
        	if {[file mtime $root/sizes.xml] < [file mtime $item]} {
            	set todo 1
            }
        }
        puts "Processing file : $item"
		if $todo {
		    if $htmlonly {
	    	    publish 3 "-htmlonly -nohtml $item"        	
            } else {
	    	    publish 2 "-nohtml $item"        	
            }
        }
	}
    
    foreach language [exec $xsltproc $xmllab/xsl/list_languages.xsl $source] {
		file copy -force [file join $imagesdir $language\_flag.png] .
    }
    
    file copy -force $xmllab/xsl/xmlverbatim.css .

	cd $oldpwd

	# Generating the html pages
	
    exec $xsltproc 	--stringparam file $filename\
                    $xmllab/xsl/publishlist_html2.xsl $source
   

}

proc publish {argc argv} {

global env tcl_platform xmllab_bin xmllab lang TMPDIR

if !$argc {mess "usage : publish file or publish file.xml"; return 1}

if {[catch {set xmllab}]} {
	set xmllab $env(xmllab)
}

if {[catch {set lang}]} {
	set lang $env(lang)
}

set errorCode "NONE"

set k 0
set job ""
set amp &
set htmlonly 0
set nohtml 0

while {$k<$argc} {
	set arg [lindex $argv $k] 
	if {$arg=="-list"} {
        incr k
        set job [lindex $argv $k]    
        publishlist $job $htmlonly
        return 0
    } elseif {$arg=="-htmlonly"} {
    	set htmlonly 1
    } elseif {$arg=="-nohtml"} {
    	set nohtml 1
	} else {
        set job $arg	
    }
    incr k
}

if {$job=="" && $listfile==""} {
	mess "publish : missing argument"; 
	return 1
}

set rootname [file rootname $job]
set filename [file tail $rootname]

set dirname [file dirname $rootname]

if {[file pathtype $dirname]=="relative"} {
	if {$dirname=="."} {
		set dirname ""
	}
	set dirname [file join [pwd] $dirname]
}

set ext [file extension $job]

if ![file exists $job] {
	if {$ext==""} {set ext .xml}
	set source $rootname$ext
	if ![file exists $source] {mess "$job, $rootname$ext : file not found"; return 1}
} else {
	set source $job
}

if {[file writable [pwd]]} {
set outputname [file join [pwd] $filename]
    set outputdir [pwd]
} else {
	mess "You don't have write access in current directory"
    return 1
}

set xsltproc [file join $xmllab_bin xsltproc]

file mkdir $filename

set imagesdir [file join $xmllab noarch images]

foreach language [exec $xsltproc $xmllab/xsl/languages.xsl $source] {

	if !$htmlonly {
	    xmllab 6 "-appname $filename$language -screenshots -lang $language $source"
        set k 1

        set done 0
        tkwait variable done

        # finishing the screenshots

	    foreach windowname [exec $xsltproc --stringparam lang $language $xmllab/xsl/scilab_windows.xsl $source] {
    	    set image $filename\_$language\_$k
            exec convert $image.gif $filename/$image.png
            exec convert -resize 75%x75%  $image.gif $filename/$image\_small.png 
            file delete $image.gif
    	    incr k
        }

        set image $filename\_$language\_0   
        file copy -force $image.png $filename/
        exec convert -resize 75%x75%  $filename/$image.png $filename/$image\_small.png
        file delete -force $image.png

        file copy -force [file join $imagesdir $language\_flag.png] $filename

	}	

    # Making the .sce archives
    
    set archiveB $filename/$filename\_$language\_XMLlabBundle.sce
    set archive $filename/$filename\_$language.sce

    
    if !$htmlonly { 
        file copy -force [file join $xmllab publishing XMLlab.zip] $archiveB

        # Archive with bundled XMLlab runtime

        xmllab 3 "-lang $language $source"
        exec zip -rqm $archiveB $filename.sce $filename.tk [exec $xsltproc $xmllab/xsl/images.xsl $source]     

        # Regular archive (needs XMLlab on the client side)

        xmllab 4 "-pack -lang $language $source"
        file delete -force $archive
		exec zip -rqj $archive $source
        exec zip -rqm $archive $filename.sce  [exec $xsltproc $xmllab/xsl/images.xsl $source]     
	}    

    set runtimesize [expr [file size $archiveB]/1024] 
    set noruntimesize [expr [file size $archive]/1024] 

	set sizesfile [open $filename/sizes.xml w]
    puts $sizesfile "<?xml version=\"1.0\"?>"
	puts $sizesfile "<sizes regular=\"$noruntimesize\" bundle=\"$runtimesize\"/>"
	close $sizesfile
}



# Generating the html pages

if !$nohtml {
	exec $xsltproc --stringparam file $filename\
                  --stringparam runtimesize $runtimesize\
                  --stringparam noruntimesize $noruntimesize\
                  $xmllab/xsl/publish_html2.xsl $source
}

# Copying the source HTML view of the XML file

exec $xsltproc --novalid\
              --stringparam "css-stylesheet" "../xmlverbatim.css"\
			  -o $filename/$filename.html\
               $xmllab/xsl/xmlverbatimwrapper.xsl $source

# Copying the sourceXML file

file copy -force $source $filename/

return 0
}

