proc xmllab {argc argv} {

global env tcl_platform xmllab_bin xmllab lang TMPDIR

if !$argc {mess "usage : xmllab file or xmllab file.xml"; return 1}

if {[catch {set xmllab}]} {
	set xmllab $env(xmllab)
}

if {[catch {set lang}]} {
	set lang $env(lang)
}

set errorCode "NONE"

set run 0
set valid 0
set k 0
set job ""

while {$k<$argc} {
	set arg [lindex $argv $k] 
	if {$arg=="-lang"} {
        incr k
        set lang [lindex $argv $k]
    } elseif {$arg=="-run"} {
    	set run 1
    } elseif {$arg=="-valid"} {
    	set valid 1
    } else {
        set job $arg	
    }
    incr k
}

if {$job==""} {
	mess "xmllab : missing argument"; 
	return 1
}

set rootname [file rootname $job]
set filename [file tail $rootname]

set dirname [file dirname $rootname]

if {[file pathtype $dirname]=="relative"} {
	if {$dirname=="."} {
		set dirname ""
	}
	set dirname [file join [pwd] $dirname]
}

set ext [file extension $job]

if ![file exists $job] {
	if {$ext==""} {set ext .xml}
	set source $rootname$ext
	if ![file exists $source] {mess "$job, $rootname$ext : file not found"; return 1}
} else {
	set source $job
}

if $run {
	set outputname [file join $TMPDIR $filename]
       	set outputdir $TMPDIR
} else {
    if {[file writable [pwd]]} {
	set outputname [file join [pwd] $filename]
        set outputdir [pwd]
    } else {
		mess "You don't have write access in current directory"
        return 1
    }
}
set xsltproc [file join $xmllab_bin xsltproc]
set xmllint [file join $xmllab_bin xmllint]


set fileversion [exec $xsltproc $xmllab/xsl/fileversion.xsl $source]

if {$fileversion=="1.2"} {
    if {[catch {exec $xsltproc --novalid --stringparam file $filename $xmllab/xsl/convert1.3.xsl $source > $outputname-converted.xml} mess]} {
	    mess $mess
	    if {$errorCode!="NONE"} {
		    return 1;
	    }
    }
	file copy -force $rootname.xml $outputname.xml.1.2
	file rename -force $outputname-converted.xml $outputname.xml	
    set fileversion "1.3"
    set source $outputname.xml
}

if {$fileversion == "1.3"} {
    if {[catch {exec $xsltproc --novalid --stringparam file $filename $xmllab/xsl/convert1.4.xsl $source > $outputname-converted.xml} mess]} {
	    mess $mess
	    if {$errorCode!="NONE"} {
		    return 1;
	    }
    }
	file copy -force $rootname.xml  $outputname.xml.1.3
	file rename -force  $outputname-converted.xml  $outputname.xml	
    set fileversion "1.4"
    set source $outputname.xml
}

# Validation

if {[catch {exec $xmllint -noout --valid $source} mess]} {
	mess $mess
	if {$errorCode!="NONE"} {
		return 1;
	}
}

if $valid { # Validate only
	return 0
}


if {[catch {exec $xsltproc --stringparam lang $lang\
                      --stringparam os $tcl_platform(os)\
					   --stringparam file $filename\
					   --stringparam rootname $rootname\
					    $xmllab/xsl/simulation.xsl\
						 $source > $outputname.sciml} mess]} {
	mess $mess
	if {$errorCode!="NONE"} {
		return 1;
	}
}


if [file exists $rootname-converted.xml] {
	file copy -force $rootname.xml $rootname.xml.old
	file rename -force $rootname-converted.xml $rootname.xml	
}

if {[catch {exec  $xsltproc $xmllab/xsl/sciml.xsl $outputname.sciml > $outputname.sce} mess]} {
	mess $mess
	if {$errorCode!="NONE"} {
		return 1;
	}
}

exec $xsltproc --stringparam lang $lang\
               --stringparam os $tcl_platform(os)\
	       --stringparam file $filename\
	       --stringparam dirname $dirname\
	       --stringparam sourcename $source\
	       --stringparam outputdir $TMPDIR\
	        $xmllab/xsl/simultk.xsl $source > $outputname.tksci
	       
exec $xsltproc --stringparam file $filename --stringparam dirname $dirname $xmllab/xsl/tk.xsl $outputname.tksci > $outputname.tk

# images to be copied side by side with the sce and tk files

set images [exec $xsltproc $xmllab/xsl/images.xsl $source]

if {[string length $images]>0} {
	foreach file $images {
		catch {file copy -force [file join $dirname $file] [file join $outputdir $file]}
	}
}

if {[file exists [file join $dirname $filename.sci] ]} {	
	file copy -force [file join $dirname $filename.sci] [file join $outputdir $filename.sci]
}

if {[file exists [file join $dirname $filename.mod] ]} {
	file copy -force [file join $dirname $filename.mod] [file join $outputdir $filename.mod]
}

file delete $outputname.sciml
file delete $outputname.tksci

if $argc==1 {
	return 0
}

if $run {
	if {[catch {ScilabEval 0}]} { # If we are outside Scilab
	    switch $tcl_platform(os) {
		    "Windows NT" {catch {exec cmd /c start /min $env(SCILAB)\\bin\\scilex -nwni -nb -f $outputname.sce &} mess}
		    "Linux" {exec scilab -nwni -nb -f $outputname.sce > /dev/null &}
		    "SunOs" {exec scilab -nwni -nb -f $outputname.sce > /dev/null &} 
		    "Darwin" {exec scilab -nwni -nb -f $outputname.sce >  /dev/null &}
	    }
    } else { # If we were called from Scilab
		ScilabEval exec(\"$outputname.sce\",-1)
    }
}
return 0
}
