<?xml version="1.0"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
xmlns:str="http://exslt.org/strings"
exclude-result-prefixes="str">

<xsl:output method="xml" indent="no" encoding="ISO-8859-1"/>

<xsl:key name="point" match="point" use="@label"/>


<xsl:strip-space elements="*"/>

<!-- Generation of code for display -->

<xsl:template match="display" mode="functions">
	<macro-call name="graphinit">
		<inputs>
			<parm>
				<xsl:choose>
					<xsl:when test="@background='white'">
						<xsl:text>[.9,.9,.9]</xsl:text>
					</xsl:when>
					<xsl:when test="@background='black'">
						<xsl:text>[0.1,0.1,0.1]</xsl:text>
					</xsl:when>
					<xsl:when test="@background='blue'">
						<xsl:text>[0,0,0.5]</xsl:text>
					</xsl:when>
				</xsl:choose>
			</parm>
		</inputs>
	</macro-call>
	<xsl:apply-templates mode="functions"/>	
</xsl:template>

<xsl:template match="display" mode="script">
	<xsl:apply-templates/>	
</xsl:template>

<xsl:template match="window" mode="functions">
	<macro-call name="fig">
		<inputs>
			<parm>
				<xsl:value-of select="position()-1"/>
			</parm>
			<parm><string>margin</string></parm>
 			<parm>0.09</parm>
		</inputs>
	</macro-call>
	<xsl:if test="@colormap">
		<macro-call name="colormap">
			<inputs>
				<parm>
					<function-call name="{@colormap}colormap">
						<inputs>
							<parm><xsl:value-of select="$colormapsize"/></parm>
						</inputs>
					</function-call>
				</parm>
			</inputs>
		</macro-call>
	</xsl:if>
   	 
<!--    <macro-call name="clf"/>-->

	<macro-call name="xset">
			<inputs>
				<parm>
					<string>wpdim</string>
				</parm>
				<parm><xsl:value-of select="concat('450+256*(',@splitx,'-1)')"/></parm>
				<parm><xsl:value-of select="concat('360+224*(',@splity,'-1)')"/></parm>
			</inputs>
	</macro-call>
	<macro-call name="xset">
			<inputs>
				<parm>
					<string>wresize</string>
				</parm>
				<parm>1</parm>
			</inputs>
	</macro-call>

	<macro-call name="xname">
		<inputs>
			<parm>
				<string>
                	<xsl:value-of select="normalize-space(title[@lang=$lang])"/>
				</string>
			</parm>
		</inputs>
	</macro-call>
    
	<macro-call name="xset">
		<inputs>
			<parm>
				<string>pixmap</string>
			</parm>
            <parm>1</parm>
		</inputs>
	</macro-call>
    

</xsl:template>

<xsl:template match="window">
	<xsl:choose>
		<xsl:when test="@dependencies">
			<ifthenelse>
				<condition>
					<function-call name="or">
						<inputs>
							<parm>
								<list sep=",">
									<xsl:for-each select="str:split(@dependencies,' ')">
										<parm>
											<xsl:value-of select="concat(.,'_changed')"/>
										</parm>
									</xsl:for-each>
								</list>
							</parm>
						</inputs>
					</function-call>
				</condition>
				<if>
					<xsl:call-template name="window"/>
				</if>
			</ifthenelse>
		</xsl:when>
		<xsl:otherwise>
			<xsl:call-template name="window"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="window">
	
	<xsl:if test="count(../window)>1">
		<macro-call name="fig">
			<inputs>
				<parm>
					<xsl:value-of select="position()-1"/>
				</parm>
			</inputs>
		</macro-call>
	</xsl:if>

	<assign>
    	<lhs>_wdim</lhs>
        <rhs>
			<function-call name="xget">
            	<inputs>
                	<parm>
                    	<string>wdim</string>
                    </parm>
                </inputs>
            </function-call>
       </rhs>
    </assign>        

	<macro-call name="clf"/>

	<xsl:for-each select="axis2d|axis3d">

		<xsl:if test="last()>1">
			<macro-call name="subplot">
				<inputs>
					<parm>
						<xsl:value-of select="../@splity"/>
					</parm>
					<parm>
						<xsl:value-of select="../@splitx"/>
					</parm>
					<parm>
						<xsl:value-of select="position()"/>
					</parm>
				</inputs>
			</macro-call>
		</xsl:if>

		<xsl:if test="@cmin and @cmax">
			<macro-call name="caxis">
				<inputs>
					<parm>
						<list>
							<parm>
								<xsl:value-of select="@cmin"/>
							</parm>
							<parm>
								<xsl:value-of select="@cmax"/>
							</parm>
						</list>
					</parm>
				</inputs>
			</macro-call>
		</xsl:if>
				
		<xsl:apply-templates mode="plot" select="*[not(self::drawpoints)]"/>

		<xsl:apply-templates mode="text" />

		<xsl:apply-templates mode="plot" select="drawpoints"/>

		<xsl:apply-templates mode="labels" select="*[1]"/>

		<macro-call name="legend">
			<inputs>
				<xsl:apply-templates mode="legend"/>
			</inputs>
		</macro-call>

		<xsl:if test="@cmin and @cmax">
			<macro-call name="caxis">
				<inputs>
					<parm>
						<string>auto</string>
					</parm>
				</inputs>
			</macro-call>
		</xsl:if>
		
	</xsl:for-each>	

	<macro-call name="xset">
		<inputs>
			<parm>
				<string>wshow</string>
			</parm>
		</inputs>
	</macro-call>

	<macro-call name="set_posfig_dim">
		<inputs>
			<parm>_wdim(1)</parm>
			<parm>_wdim(2)</parm>
		</inputs>
	</macro-call>

	<xsl:if test="$screenshots='yes'">
	    <macro-call name="xs2gif">
		    <inputs>
				<parm>
					<xsl:value-of select="position()-1"/>                	
                </parm>
			    <parm>
				    <string>
                    	<xsl:value-of select="concat($file,'_',$lang,'_',position(),'.gif')"/>
                    </string>
			    </parm>
		    </inputs>
	    </macro-call>    
    </xsl:if>
				
</xsl:template>

<xsl:template name="add-axis-options-2d">
	<xsl:if test="(../@xmin) and (../@xmax) and  (../@ymin) and (../@ymax)">
		<parm>
			<string><verb>axis</verb></string>
		</parm>
		<parm>
			<list>
				<parm>
					<xsl:value-of select="../@xmin"/>
				</parm>
				<parm>
					<xsl:value-of select="../@xmax"/>
				</parm>
				<parm>
					<xsl:value-of select="../@ymin"/>
				</parm>
				<parm>
					<xsl:value-of select="../@ymax"/>
				</parm>
			</list>
		</parm>
		<parm>
			<string><verb>axis</verb></string>
		</parm>
		<parm>
			<string><verb>tightX</verb></string>
		</parm>
		<parm>
			<string><verb>axis</verb></string>
		</parm>
		<parm>
			<string><verb>tightY</verb></string>
		</parm>
	</xsl:if>
	<xsl:if test="../@iso='yes'">
		<parm>
			<string><verb>axis</verb></string>
		</parm>
		<parm>
			<string><verb>equal</verb></string>
		</parm>
	</xsl:if>
	<parm>
		<string><verb>axis</verb></string>
	</parm>
	<parm>
		<string><xsl:value-of select="../@position"/></string>
	</parm>
</xsl:template>

<xsl:template name="add-specific-options-3d">	
	<xsl:if test="@mode != 'wireframe'">
	       <parm>
		       <string>colorbar</string>
	       </parm>
	       <parm>
		       <string><xsl:value-of select="../@colorbar"/></string>
	       </parm>
	       <parm>
		       <string>shading</string>
	       </parm>
	       <parm>
		       <string><xsl:value-of select="@shading"/></string>
	       </parm>
	   </xsl:if>
           <xsl:apply-templates select="//*[current()/@ref=@label]" mode="labels"/>
</xsl:template>


<xsl:template name="add-axis-options-3d">
	<xsl:if test="../@iso='yes'">
		<parm>
			<string><verb>axis</verb></string>
		</parm>
		<parm>
			<string><verb>vis3d</verb></string>
		</parm>
	</xsl:if>
	<xsl:if test="(../@xmin) and (../@xmax) and  (../@ymin) and (../@ymax) and  (../@zmin) and (../@zmax)">
		<parm>
			<string><verb>axis</verb></string>
		</parm>
		<parm>
			<list>
				<parm>
					<xsl:value-of select="../@xmin"/>
				</parm>
				<parm>
					<xsl:value-of select="../@xmax"/>
				</parm>
				<parm>
					<xsl:value-of select="../@ymin"/>
				</parm>
				<parm>
					<xsl:value-of select="../@ymax"/>
				</parm>
				<parm>
					<xsl:value-of select="../@zmin"/>
				</parm>
				<parm>
					<xsl:value-of select="../@zmax"/>
				</parm>
				
			</list>
		</parm>
	</xsl:if>
	<xsl:if test="@hidden">
		<parm>
			<string><verb>hidden</verb></string>
		</parm>
		<parm>
			<string><xsl:value-of select="@hidden"/></string>
		</parm>
	</xsl:if>

	<parm>
		<string>view</string>
	</parm>
	<parm>
		<list>
			<parm><xsl:value-of select="../@azimuth"/></parm>
			<parm><xsl:value-of select="../@elevation"/></parm>
		</list>
	</parm>

</xsl:template>

<xsl:template name="hold-if-necessary">

	<xsl:if test="position()=1">
		<macro-call name="hold">
			<inputs>
				<parm>
					<string>on</string>
				</parm>
			</inputs>
		</macro-call>
	</xsl:if>
	
	<xsl:if test="position()=last()">
		<macro-call name="hold">
			<inputs>
				<parm>
					<string>off</string>
				</parm>
			</inputs>
		</macro-call>
	</xsl:if>

</xsl:template>


<xsl:template name="triangulation-test">
		<xsl:for-each select="//*[current()/@ref=@label]">
			<xsl:for-each select="defdomain2d | //defdomain2d[@label=current()/refdomain2d/@ref] | //defdomain2d[@label=current()/../refdomain2d/@ref]">
				<xsl:if test="border">
					<xsl:value-of select="'tri'"/>
				</xsl:if>
			</xsl:for-each>		
		</xsl:for-each>
</xsl:template>

<xsl:template match="drawpoints" mode="plot">
	<macro-call name="init_target">
		<inputs>
			<parm>
				<list sep=";">
                	<xsl:call-template name="split-point-ref">
                    	<xsl:with-param name="content" select="@ref"/>
                    </xsl:call-template>
				</list>
			</parm>
		</inputs>
	</macro-call>
</xsl:template>

<xsl:template name="init_target_parms">
   <parm>
	  <string>
		  <xsl:value-of select="x1/@label"/>
	  </string>
	  <xsl:text>,</xsl:text>
	  <string>
		  <xsl:value-of select="x2/@label"/>
	  </string>
	  <xsl:text>,</xsl:text>
	  <string>
		  <xsl:value-of select="@label"/>
	  </string>							
   </parm>
</xsl:template>

<xsl:template name="split-point-ref">
  <xsl:param name="content"/>
  <xsl:choose>
    <xsl:when test="contains($content, ' ')">
      <xsl:for-each select="key('point',substring-before($content, ' '))">
        <xsl:call-template name="init_target_parms"/>
      </xsl:for-each>
      <xsl:call-template name="split-point-ref">
        <xsl:with-param name="content" select="substring-after($content, ' ')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:for-each select="key('point',$content)">
        <xsl:call-template name="init_target_parms"/>
      </xsl:for-each>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>


<xsl:template match="drawcurve2d" mode="plot">
	<macro-call name="plot">
		<inputs>
			<xsl:apply-templates select="//*[current()/@ref=@label]" mode="plot"/>
			<parm>
				<string>
					<xsl:choose>
						<xsl:when test="@color='blue'">
							<verb>b</verb>
						</xsl:when>
						<xsl:when test="@color='yellow'">
							<verb>y</verb>
						</xsl:when>
						<xsl:when test="@color='green'">
							<verb>g</verb>
						</xsl:when>
						<xsl:when test="@color='red'">
							<verb>r</verb>
						</xsl:when>
						<xsl:when test="@color='cyan'">
							<verb>c</verb>
						</xsl:when>
						<xsl:when test="@color='magenta'">
							<verb>m</verb>
						</xsl:when>
						<xsl:when test="@color='black'">
							<verb>k</verb>
						</xsl:when>
					</xsl:choose>
					<xsl:choose>
						<xsl:when test="@marker='point'">
							<verb>.</verb>
						</xsl:when>
						<xsl:when test="@marker='circle'">
							<verb>o</verb>
						</xsl:when>
						<xsl:when test="@marker='trefoil'">
							<verb>t</verb>
						</xsl:when>
						<xsl:when test="@marker='plus'">
							<verb>+</verb>
						</xsl:when>
						<xsl:when test="@marker='star'">
							<verb>*</verb>
						</xsl:when>
						<xsl:when test="@marker='triangle-up'">
							<verb>^</verb>
						</xsl:when>
						<xsl:when test="@marker='triangle-down'">
							<verb>v</verb>
						</xsl:when>
						<xsl:when test="@marker='diamond'">
							<verb>d</verb>
						</xsl:when>
						<xsl:when test="@marker='filled-diamond'">
							<verb>f</verb>
						</xsl:when>						
					</xsl:choose>
					<xsl:choose>
						<xsl:when test="@linetype='dashed'">
							<verb>--</verb>
						</xsl:when>
						<xsl:when test="@linetype='dashdot'">
							<verb>-.</verb>
						</xsl:when>
						<xsl:when test="@linetype='dot'">
							<verb>.</verb>
						</xsl:when>
						<xsl:when test="@linetype='solid'">
							<verb>-</verb>
						</xsl:when>
					</xsl:choose>
					<verb>
						<xsl:value-of select="@thickness"/>
					</verb>
				</string>
			</parm>
			<xsl:call-template name="add-axis-options-2d"/>
		</inputs>
	</macro-call>

	<xsl:call-template name="hold-if-necessary"/>
	
</xsl:template>


<xsl:template match="drawcurve2d|drawcurve3d" mode="legend">
	<xsl:for-each select="//*[current()/@ref=@label]">
		<parm>
			<string>
				<xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
			</string>
		</parm>
	</xsl:for-each>
</xsl:template>

<xsl:template match="drawcurve2d" mode="labels">
	<xsl:apply-templates select="//*[current()/@ref=@label]" mode="labels"/>
</xsl:template>

<xsl:template match="drawcurve2d|drawcurve3d" mode="text">
	<xsl:apply-templates select="//*[current()/@ref=@label]" mode="text"/>
</xsl:template>

<xsl:template match="drawsurface" mode="text">
	<xsl:apply-templates select="//*[current()/@ref=@label]" mode="text"/>
</xsl:template>


<xsl:template match="drawsurface" mode="legend">
	<xsl:for-each select="//*[current()/@ref=@label]">
		<parm>
			<string>
				<xsl:value-of select="normalize-space(x3/name[(@lang=$lang) or (not(@lang))])"/>
			</string>
		</parm>
	</xsl:for-each>
</xsl:template>

<xsl:template match="axis2d/drawsurface" mode="plot">

	<xsl:variable name="triangulation">
		<xsl:call-template name="triangulation-test"/>
	</xsl:variable>
	
	<macro-call name="{$triangulation}pcolor">
		<inputs>
			<xsl:apply-templates select="//*[current()/@ref=@label]" mode="plot"/>
			<xsl:call-template name="add-axis-options-2d"/>
			<xsl:call-template name="add-specific-options-3d"/>			
		</inputs>
	</macro-call>

	<xsl:call-template name="hold-if-necessary"/>

</xsl:template>

<xsl:template match="axis3d/drawsurface" mode="plot">

	<xsl:variable name="triangulation">
		<xsl:call-template name="triangulation-test"/>
	</xsl:variable>

	<xsl:variable name="macroname">
		<xsl:choose>
			<xsl:when test="@mode='level'">
				<xsl:value-of select="concat($triangulation,'surf')"/>
			</xsl:when>
			<xsl:when test="@mode='light'">
				<xsl:value-of select="concat($triangulation,'surfl')"/>
			</xsl:when>
			<xsl:when test="@mode='wireframe'">
				<xsl:value-of select="concat($triangulation,'mesh')"/>
			</xsl:when>
		</xsl:choose>
	</xsl:variable>

	<macro-call name="{$macroname}">
		<inputs>
			<xsl:apply-templates select="//*[current()/@ref=@label]" mode="plot"/>
			<xsl:call-template name="add-axis-options-3d"/>
			<xsl:call-template name="add-specific-options-3d"/>
		</inputs>
	</macro-call>

	<xsl:call-template name="hold-if-necessary"/>

</xsl:template>

<xsl:template match="drawcurve3d" mode="plot">
	<macro-call name="plot3">
		<inputs>
			<xsl:apply-templates select="//*[current()/@ref=@label]" mode="plot"/>
			<parm>
				<string>
					<xsl:choose>
						<xsl:when test="@color='blue'">
							<verb>b</verb>
						</xsl:when>
						<xsl:when test="@color='yellow'">
							<verb>y</verb>
						</xsl:when>
						<xsl:when test="@color='green'">
							<verb>g</verb>
						</xsl:when>
						<xsl:when test="@color='red'">
							<verb>r</verb>
						</xsl:when>
						<xsl:when test="@color='cyan'">
							<verb>c</verb>
						</xsl:when>
						<xsl:when test="@color='magenta'">
							<verb>m</verb>
						</xsl:when>
						<xsl:when test="@color='black'">
							<verb>k</verb>
						</xsl:when>
					</xsl:choose>
					<xsl:choose>
						<xsl:when test="@marker='point'">
							<verb>.</verb>
						</xsl:when>
						<xsl:when test="@marker='circle'">
							<verb>o</verb>
						</xsl:when>
						<xsl:when test="@marker='trefoil'">
							<verb>t</verb>
						</xsl:when>
						<xsl:when test="@marker='plus'">
							<verb>+</verb>
						</xsl:when>
						<xsl:when test="@marker='star'">
							<verb>*</verb>
						</xsl:when>
						<xsl:when test="@marker='triangle-up'">
							<verb>^</verb>
						</xsl:when>
						<xsl:when test="@marker='triangle-down'">
							<verb>v</verb>
						</xsl:when>
						<xsl:when test="@marker='diamond'">
							<verb>d</verb>
						</xsl:when>
						<xsl:when test="@marker='filled-diamond'">
							<verb>f</verb>
						</xsl:when>						
					</xsl:choose>
					<xsl:choose>
						<xsl:when test="@linetype='dashed'">
							<verb>--</verb>
						</xsl:when>
						<xsl:when test="@linetype='dashdot'">
							<verb>-.</verb>
						</xsl:when>
						<xsl:when test="@linetype='dot'">
							<verb>.</verb>
						</xsl:when>
						<xsl:when test="@linetype='solid'">
							<verb>-</verb>
						</xsl:when>
					</xsl:choose>
					<verb>
						<xsl:value-of select="@thickness"/>
					</verb>
				</string>
			</parm>
			<xsl:call-template name="add-axis-options-3d"/>
			<xsl:call-template name="add-specific-options-3d"/>
		</inputs>
	</macro-call>

	<xsl:call-template name="hold-if-necessary"/>
	
</xsl:template>

<xsl:template match="title"/>
<xsl:template match="*" mode="text"/>


</xsl:stylesheet>
