<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet xmlns:exslt="http://exslt.org/common" xmlns:str="http://exslt.org/strings" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" exclude-result-prefixes="exslt str">

  <xsl:param name="file"/>
  <xsl:param name="rootname"/>
  <xsl:param name="language"/>
  <xsl:param name="screenshots"/>
  <xsl:param name="packsource"/>

  <xsl:variable name="lang">
  	<xsl:choose>
    	<xsl:when test="$language='DEFAULT'">
        	<xsl:value-of select="/simulation/@default-lang"/>
        </xsl:when>
        <xsl:otherwise>
        	<xsl:value-of select="$language"/>
        </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

<!--  <xsl:variable name="version">
    <xsl:value-of select="document('version.xml')/xmllab/@version"/>
  </xsl:variable>-->

  <xsl:variable name="colormapsize">128</xsl:variable>

  <xsl:include href="localization.xsl"/>

  <xsl:output method="xml" indent="no" encoding="ISO-8859-1"/>

  <xsl:strip-space elements="*"/>

  <xsl:template match="/">
    <xsl:choose>
    	<xsl:when test="$packsource='yes'">
        	<xsl:apply-templates mode="packsource"/>
        </xsl:when>
        <xsl:otherwise>
    		<xsl:apply-templates/>
        </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="no-xmllab-message">
    <macro-call name="x_message">
      <inputs>
        <parm>
          <list sep=";">
            <parm>
              <string>You need the XMLlab toolbox version &gt;= <xsl:value-of select="$version"/></string>
            </parm>
            <parm>
              <string>to run this simulation.</string>
            </parm>
            <parm>
              <string/>
            </parm>
            <parm>
              <string>Please visit http://xmllab.org</string>
            </parm>
          </list>
        </parm>
      </inputs>
    </macro-call>
    <macro-call name="quit"/>
  </xsl:template>

  <xsl:template match="simulation" mode="packsource">
      <sciml>
        <ifthenelse>
          <condition>
                <function-call name="~exists">
                  <inputs>
                    <parm>
                      <string>%xmllab</string>
                    </parm>
                  </inputs>
                </function-call>
          </condition>
          <if>
            <xsl:call-template name="no-xmllab-message"/>
          </if>
          <else>
                        <macro-call name="xmllab">
                            <inputs>
                                <parm><string>-lang</string></parm>
                                <parm><string><xsl:value-of select="$lang"/></string></parm>
                                <parm><string>-run</string></parm>
                                <parm><string><xsl:value-of select="$file"/></string></parm>
                            </inputs>
                        </macro-call>
          </else> 
        </ifthenelse>
      </sciml>
  </xsl:template>

  <xsl:template match="simulation">  <!-- This main template generates the sciml code for
                                          generating the main Scilab script that defines
                                          auxilliary functions (ode's and implicit functions rhs,
                                          and others) and the principal callback function "run()"
                                          which proceeds to computations and display. -->
    <sciml>
        
      <function-definition name="proceed">
        <body>
        
          <xsl:if test="//point">
            <macro-call name="seteventhandler">
              <inputs>
                <parm>
                  <string/>
                </parm>
              </inputs>
            </macro-call>
          </xsl:if>
        
          <xsl:apply-templates mode="script"/>
        
          <xsl:if test="//point">
        
            <macro-call name="seteventhandler">
              <inputs>
                <parm>
                  <string>xmllab_handler</string>
                </parm>
              </inputs>
            </macro-call>
        
          </xsl:if>
        
        </body>
      </function-definition>


      <function-definition name="main">
        <body>

        <ifthenelse>
          <condition>
            <function-call name="~exists">
              <inputs>
                <parm>
                  <string>%xmllab</string>
                </parm>
              </inputs>
            </function-call>
          </condition>
          <if>
        	  <ifthenelse>
            	  <condition>
                	  <function-call name="isdir">
            			  <inputs><parm><string>XMLlab</string></parm></inputs>
          			  </function-call>
                  </condition>
                  <if>
					  <comment>Load the runtime XMLlab library</comment>
                	  <macro-call name="exec">
                    	  <inputs>
                        	  <parm>
                            	  <string>XMLlab/runtime_loader.sce</string>
                              </parm>
                          </inputs>
                      </macro-call>
                  </if>
 				  <else>
                    <xsl:call-template name="no-xmllab-message"/>
				  </else>
              </ifthenelse>
		  </if>
        </ifthenelse>

        <xsl:apply-templates mode="functions"/>

        <assign>
          <lhs>first_run</lhs>
          <rhs>%t</rhs>
        </assign>

        <xsl:copy-of select="exslt:node-set($strings)/credits[@lang=$lang]"/>


  <!-- At last, we launch the TK interface code -->

        <assign>
          <lhs>oldpwd</lhs>
          <rhs>
            <function-call name="pwd"/>
          </rhs>
        </assign>

        <macro-call name="chdir">
          <inputs>
            <parm>
              <function-call name="get_absolute_file_path">
                <inputs>
                  <parm>
                    <string>
                      <xsl:value-of select="concat($file,'.sce')"/>
                    </string>
                  </parm>
                </inputs>
              </function-call>
            </parm>
          </inputs>
        </macro-call>

        <macro-call name="TCL_EvalFile">
          <inputs>
            <parm>
              <string>
                <xsl:value-of select="$file"/>
                <verb>.tk</verb>
              </string>
            </parm>
          </inputs>
        </macro-call>

        <macro-call name="chdir">
          <inputs>
            <parm>oldpwd</parm>
          </inputs>
        </macro-call>

        <xsl:text>exec(proceed,-1);</xsl:text>
        <xsl:text>TCL_EvalStr('set ok 1');</xsl:text>

        <assign>
          <lhs>first_run</lhs>
          <rhs>%f</rhs>
        </assign>

        <macro-call name="TCL_EvalStr">
          <inputs>
            <parm>
              <string>raise $toplevelWindow</string>
            </parm>
          </inputs>
        </macro-call>

	    <xsl:choose>

      	  <xsl:when test="$screenshots='yes'">

        	  <macro-call name="unix_g">
            	  <inputs>
                	  <parm>
                    	  <string>
	                    	  <xsl:value-of select="concat('import -window &quot;&quot;',normalize-space(header/title[@lang=$lang]),' (0)&quot;&quot; ',$file,'_',$lang,'_0.png')"/>
                    	  </string>
                      </parm>
                  </inputs>
              </macro-call>
        	  <macro-call name="unix_g">
            	  <inputs>
                	  <parm>
                    	  <string>
	                    	  <xsl:value-of select="concat('import -window &quot;&quot;',normalize-space(header/title[@lang=$lang]),' (0)&quot;&quot; ',$file,'_',$lang,'_0.png')"/>
                    	  </string>
                      </parm>
                  </inputs>
              </macro-call>
              <macro-call name="TCL_EvalStr">
			    <inputs>
            	  <parm>
                    <string>send publish set done 1</string>
                  </parm>
                </inputs>
              </macro-call>    
			  <macro-call name="quit"/>

          </xsl:when>

          <xsl:otherwise>

            <xsl:text>while TCL_GetVar('keep')=='yes';realtimeinit(1/1000);realtime(0);realtime(1);end;</xsl:text>
            <xsl:text>TCL_EvalStr('catch {bind $toplevelWindow &lt;Destroy&gt; {}}; catch {destroy $toplevelWindow}');</xsl:text>
            <xsl:text>if(or(sciargs()=="-f")) quit; end;</xsl:text>

            <xsl:if test="//point">
              <xsl:text>seteventhandler('');</xsl:text>
            </xsl:if>

            <macro-call name="chdir">
              <inputs>
                <parm>oldpwd</parm>
              </inputs>
            </macro-call>        

          </xsl:otherwise>
        </xsl:choose>
	</body>
   </function-definition>
    
    <macro-call name="exec">
    	<inputs>
        	<parm>main</parm>
        	<parm>-1</parm>
        </inputs>
    </macro-call>
    
    </sciml>
  
  </xsl:template>
  
  <xsl:template match="header" mode="script">
    <xsl:apply-templates select="title" mode="script"/>
  </xsl:template>
  
  <xsl:template match="header" mode="functions"/>
  
  <xsl:template match="title|name" mode="script">
     <comment>
    	<xsl:if test="@lang=$lang or not(@lang)">
        	<xsl:value-of select="normalize-space(.)"/>
        </xsl:if>
      </comment>
  </xsl:template>
    
  <xsl:template match="name" mode="functions"/>
  <xsl:template match="name" mode="script"/>
  <xsl:template match="title" mode="functions"/>
  <xsl:template match="title" mode="script"/>
  <xsl:template match="image" mode="functions"/>
  <xsl:template match="image" mode="script"/>
  
  <xsl:template match="section" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>
  
  <xsl:template match="subsection" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>
  
  <xsl:template match="section" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="subsection" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="section[@hidden='yes']" mode="script"/>
  <xsl:template match="subsection[@hidden='yes']" mode="script"/>
  <xsl:template match="section[@hidden='yes']" mode="functions"/>
  <xsl:template match="subsection[@hidden='yes']" mode="functions"/>
  
  <xsl:template match="database" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>
  
  <xsl:template match="database" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="record" mode="script"/>
  <xsl:template match="record" mode="functions"/>
  
  <xsl:template match="parameters" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="parameters" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>
  
  <xsl:template match="matrix" mode="script">

    <macro-call name="TCL_EvalStr">
      <inputs>
        <parm>
          <string>
            <xsl:text>set s ""\[;[ $toplevelWindow.</xsl:text>
            <xsl:value-of select="@label"/>
            <xsl:text> selection set origin end; selection get ];\]""; </xsl:text>
            <xsl:text>while {[regsub -all "",,"" $s "",0,"" s]} {};</xsl:text>
            <xsl:text>while {[regsub -all "";,"" $s "";0,"" s]} {};</xsl:text>
            <xsl:text>while {[regsub -all "",;"" $s "",0;"" s]} {};</xsl:text>
            <xsl:text>$toplevelWindow.</xsl:text>
            <xsl:value-of select="@label"/>
            <xsl:text> selection clear all;</xsl:text>
            <xsl:text>set P</xsl:text>
            <xsl:value-of select="@label"/>
            <xsl:text> $s;</xsl:text>
          </string>
        </parm>
      </inputs>
    </macro-call>
  
    <assign>
      <lhs>
        <list>
          <parm>
            <xsl:value-of select="@label"/>
          </parm>
          <parm>ierr</parm>
        </list>
      </lhs>
      <rhs>
        <function-call name="TCL_GetMatrix">
          <inputs>
            <parm>
              <string>
                <xsl:text>P</xsl:text>
                <xsl:value-of select="@label"/>
              </string>
            </parm>
            <parm>
              <string>
                <xsl:value-of select="@stripcol"/>
              </string>
            </parm>
            <parm>
              <string>
                <xsl:value-of select="@striprow"/>
              </string>
            </parm>
          </inputs>
        </function-call>
      </rhs>
    </assign>

    <ifthenelse>
      <condition>ierr</condition>
      <if>return;</if>
    </ifthenelse>

  </xsl:template>

  <xsl:template match="group" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>

  <xsl:template match="group" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>

  <xsl:template match="scalar" mode="functions">

    <assign>
      <lhs>
        <xsl:value-of select="@label"/>
      </lhs>
      <rhs>
        <xsl:choose>
          <xsl:when test="string-length(value)&gt;0">
            <xsl:value-of select="value"/>
            <xsl:if test="@scale">
              <xsl:value-of select="concat('*',@scale)"/>
            </xsl:if>
          </xsl:when>
          <xsl:otherwise>0</xsl:otherwise>
        </xsl:choose>
      </rhs>
    </assign>

  </xsl:template>

  <xsl:template match="point" mode="functions">

    <xsl:for-each select="x1|x2|x3">
      <assign>
        <lhs>
          <xsl:value-of select="@label"/>
        </lhs>
        <rhs>
          <xsl:value-of select="value"/>
        </rhs>
      </assign>
    </xsl:for-each>

    <function-definition name="{concat('update_',@label)}">
      <inputs>
        <parm>old_<xsl:value-of select="x1/@label"/></parm>
        <parm>old_<xsl:value-of select="x2/@label"/></parm>
        <parm>new_<xsl:value-of select="x1/@label"/></parm>
        <parm>new_<xsl:value-of select="x2/@label"/></parm>
      </inputs>
      <outputs>
        <parm>
          <xsl:value-of select="x1/@label"/>
        </parm>
        <parm>
          <xsl:value-of select="x2/@label"/>
        </parm>
      </outputs>
      <body>
        <assign>
          <lhs>
            <xsl:value-of select="x1/@label"/>
          </lhs>
          <rhs>new_<xsl:value-of select="x1/@label"/></rhs>
        </assign>
        <assign>
          <lhs>
            <xsl:value-of select="x2/@label"/>
          </lhs>
          <rhs>new_<xsl:value-of select="x2/@label"/></rhs>
        </assign>
        <xsl:if test="constraints/curve">
          <assign>
            <lhs>
              <list sep=",">
                <parm>
                  <xsl:value-of select="x1/@label"/>
                </parm>
                <parm>
                  <xsl:value-of select="x2/@label"/>
                </parm>
              </list>
            </lhs>
            <rhs>
              <function-call name="project_on_curve">
                <inputs>
                  <parm>
                    <xsl:value-of select="x1/@label"/>
                  </parm>
                  <parm>
                    <xsl:value-of select="x2/@label"/>
                  </parm>
                  <xsl:apply-templates select="//*[current()/constraints/curve/@ref=@label]" mode="plot"/>
                </inputs>
              </function-call>
            </rhs>
          </assign>
        </xsl:if>
        <ifthenelse>
          <condition>
            <function-call name="or">
              <inputs>
                <parm>
                  <list sep=";">
                    <parm>-1</parm>
                    <xsl:for-each select="constraints/inequality | //constraints[@label=current()/refconstraints/@ref]/inequality">
                      <parm>
                        <xsl:value-of select="normalize-space(.)"/>
                      </parm>
                    </xsl:for-each>
                  </list>
                  <xsl:text>&gt;0</xsl:text>
                </parm>
              </inputs>
            </function-call>
          </condition>
          <if>
            <assign>
              <lhs>
                <xsl:value-of select="x1/@label"/>
              </lhs>
              <rhs>old_<xsl:value-of select="x1/@label"/></rhs>
            </assign>
            <assign>
              <lhs>
                <xsl:value-of select="x2/@label"/>
              </lhs>
              <rhs>old_<xsl:value-of select="x2/@label"/></rhs>
            </assign>
          </if>
        </ifthenelse>
      </body>
    </function-definition>

  </xsl:template>

  <xsl:template match="matrix" mode="functions"/>

  <xsl:template match="scalar" mode="script">
    <xsl:call-template name="TCL_GetVar"/>
  </xsl:template>

  <xsl:template match="point" mode="script">
    <xsl:for-each select="x1|x2|x3">
      <xsl:call-template name="TCL_GetVar"/>
    </xsl:for-each>
  </xsl:template>

  <xsl:template name="TCL_GetVar"> <!-- processing of parameters elements in the main script generation
                                        phase : here we generate the Scilab code that allows to convert the
                                        associated values of parameters (strings) in the TK workspace.
                                        example : a=evtsr(TCL_GetVar('a')) -->
    <assign>
      <lhs>
        <list>
          <parm>
            <xsl:value-of select="concat(@label,'_new')"/>
          </parm>
          <parm>ierr</parm>
        </list>
      </lhs>
      <rhs>
        <function-call name="evstr">
          <inputs>
            <parm>
              <function-call name="TCL_GetVar">
                <inputs>
                  <parm>
                    <string>
                      <xsl:value-of select="@label"/>
                    </string>
                  </parm>
                </inputs>
              </function-call>
              <xsl:if test="@scale">
                <xsl:value-of select="concat('+&quot;*',@scale,'&quot;')"/>
              </xsl:if>
            </parm>
          </inputs>
        </function-call>
      </rhs>
    </assign>
    
    <ifthenelse>
      <condition>ierr</condition>
      <if>return;</if>
    </ifthenelse>
    
    <assign>
      <lhs>
        <xsl:value-of select="concat(@label,'_changed')"/>
      </lhs>
      <rhs>first_run</rhs>
    </assign>
    
    <ifthenelse>
      <condition><xsl:value-of select="concat(@label,'_new')"/>~=<xsl:value-of select="@label"/></condition>
      <if>
        <assign>
          <lhs>
            <xsl:value-of select="concat(@label,'_changed')"/>
          </lhs>
          <rhs>%t</rhs>
        </assign>
      </if>
    </ifthenelse>
    
    <assign>
      <lhs>
        <xsl:value-of select="@label"/>
      </lhs>
      <rhs>
        <xsl:value-of select="concat(@label,'_new')"/>
      </rhs>
    </assign>
  
  </xsl:template>
  
  <!--	Processing of the "script" element : -->
  
  <xsl:template match="script" mode="functions"/>
  
  <xsl:template match="script[@href]" mode="script">
    <macro-call name="exec">
      <inputs>
        <parm>
          <string>
            <xsl:value-of select="@href"/>
          </string>
        </parm>
        <parm>
          <verb>-1</verb>
        </parm>
      </inputs>
    </macro-call>
  </xsl:template>
  
  <xsl:template match="script[not(@href)]" mode="script">
    <xsl:for-each select="str:split(text(),'&#10;')">
      <command>
        <xsl:value-of select="normalize-space(.)"/>
      </command>
    </xsl:for-each>
  </xsl:template>
  
<!-- Processing of the "compute" element : definition of 
     domains, high level elements such as ode's, implicit functions,
     etc. and curves.-->

  <xsl:template match="compute" mode="functions">
    <xsl:apply-templates mode="functions"/>
  </xsl:template>
  
  <xsl:template match="compute" mode="script">
    <xsl:apply-templates mode="script"/>
  </xsl:template>

<!-- These stylesheets are separated for readability -->

  <xsl:include href="domains.xsl"/>
  <xsl:include href="implicit.xsl"/>
  <xsl:include href="ode.xsl"/>
  <xsl:include href="pde.xsl"/>

  <xsl:template match="outputs"> <!-- Creation of outputs (others) -->
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="output">
    <assign>
      <lhs>
        <xsl:value-of select="@label"/>
      </lhs>
      <rhs>
        <xsl:value-of select="normalize-space(value)"/>
      </rhs>
    </assign>
  </xsl:template>

<!-- Templates for curves and surfaces -->

  <xsl:include href="curves.xsl"/>
  <xsl:include href="surfaces.xsl"/>

<!-- Templates for display -->

  <xsl:include href="display.xsl"/>
  <xsl:template name="variables">
    <xsl:for-each select="variable">
      <assign>
        <lhs>
          <xsl:value-of select="@label"/>
        </lhs>
        <rhs>
          <xsl:value-of select="normalize-space(.)"/>
        </rhs>
      </assign>
    </xsl:for-each>
  </xsl:template>

<!-- Dependencies/independencies in script mode -->

  <xsl:template match="*[@dependencies]" mode="script">
    <ifthenelse>
      <condition>
        <function-call name="or">
          <inputs>
            <parm>
              <list sep=",">
                <xsl:for-each select="str:split(@dependencies,' ')">
                  <parm>
                    <xsl:value-of select="concat(.,'_changed')"/>
                  </parm>
                </xsl:for-each>
              </list>
            </parm>
          </inputs>
        </function-call>
      </condition>
      <if>
        <xsl:apply-templates select="."/>
      </if>
    </ifthenelse>
  </xsl:template>

  <xsl:template match="*[@independencies]" mode="script">
    <ifthenelse>
      <condition>
        <or>
          <arg>first_run</arg>
          <arg>
            <not>
              <function-call name="or">
                <inputs>
                  <parm>
                    <list sep=",">
                      <xsl:for-each select="str:split(@independencies,' ')">
                        <parm>
                          <xsl:value-of select="concat(.,'_changed')"/>
                        </parm>
                      </xsl:for-each>
                    </list>
                  </parm>
                </inputs>
              </function-call>
            </not>
          </arg>
        </or>
      </condition>
      <if>
        <xsl:apply-templates select="."/>
      </if>
    </ifthenelse>
  </xsl:template>


<!-- Templates for save -->

  <xsl:template match="save" mode="script"/>

  <xsl:template match="save" mode="functions">
    <function-definition name="save_stuff">
      <body>
        <xsl:apply-templates/>
      </body>
    </function-definition>
  </xsl:template>

  <xsl:template name="get-filename">
    <function-call name="TCL_GetVar">
      <inputs>
        <parm>
          <string>file<xsl:value-of select="position()"/></string>
        </parm>
      </inputs>
    </function-call>
  </xsl:template>

  <xsl:template match="file[@format='scilab']">
    <macro-call name="save">
      <inputs>
        <parm>
          <xsl:call-template name="get-filename"/>
        </parm>
        <parm>
          <xsl:value-of select="translate(@labels,' ',',')"/>
        </parm>
      </inputs>
    </macro-call>
  </xsl:template>

  <xsl:template match="file[@format='scilab']">
    <macro-call name="save">
      <inputs>
        <parm>
          <xsl:call-template name="get-filename"/>
        </parm>
        <parm>
          <xsl:value-of select="translate(@labels,' ',',')"/>
        </parm>
      </inputs>
    </macro-call>
  </xsl:template>

  <xsl:template match="file[@format='csv']">
    <assign>
      <lhs>
        <list>
          <parm>fp</parm>
          <parm>err</parm>
        </list>
      </lhs>
      <rhs>
        <function-call name="mopen">
          <inputs>
            <parm>
              <xsl:call-template name="get-filename"/>
            </parm>
            <parm>
              <string>w</string>
            </parm>
          </inputs>
        </function-call>
      </rhs>
    </assign>
    <ifthenelse>
      <condition>~err</condition>
      <if>
        <xsl:call-template name="split-save">
          <xsl:with-param name="content" select="normalize-space(@labels)"/>
        </xsl:call-template>
        <macro-call name="mclose">
          <inputs>
            <parm>fp</parm>
          </inputs>
        </macro-call>
      </if>
    </ifthenelse>
  </xsl:template>

  <xsl:template name="split-save">
    <xsl:param name="content"/>
    <xsl:choose>
      <xsl:when test="contains($content, ' ')">
        <xsl:for-each select="//*[@label=substring-before($content, ' ')]">
          <xsl:call-template name="write-data"/>
        </xsl:for-each>
        <xsl:call-template name="split-save">
          <xsl:with-param name="content" select="substring-after($content, ' ')"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:for-each select="//*[@label=$content]">
          <xsl:call-template name="write-data"/>
        </xsl:for-each>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="write-data">
    <macro-call name="mfprintf">
      <inputs>
        <parm>fp</parm>
        <parm>
          <string>\n""%s : "",</string>
        </parm>
        <parm>
          <string>
            <xsl:value-of select="normalize-space(name[(@lang=$lang) or not(@lang)])"/>
          </string>
        </parm>
      </inputs>
    </macro-call>
    <macro-call name="mfprintf">
      <inputs>
        <parm>fp</parm>
        <parm>
          <string>%f,</string>
        </parm>
        <parm><xsl:value-of select="@label"/>(:)</parm>
      </inputs>
    </macro-call>
  </xsl:template>

  <xsl:template match="notes" mode="script"/>
  <xsl:template match="notes" mode="functions"/>
  <xsl:template match="actions" mode="script"/>
  <xsl:template match="actions" mode="functions"/>
  <xsl:template match="user-widget" mode="script"/>
  <xsl:template match="user-widget" mode="functions"/>

</xsl:stylesheet>
