<?xml version="1.0"?>

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:param name="file"/>
<xsl:param name="dirname"/>

<xsl:output method="text" indent="no" encoding="ISO-8859-1"/>

<xsl:strip-space elements="*"/>

<xsl:template match="/">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="tkml">
	<xsl:text>&#xA;encoding system iso8859-1;</xsl:text>
	<xsl:text>&#xA;global ok wait but;set ok 0;set wait 0;set but release;</xsl:text>

	<xsl:value-of select="concat('&#xA;set sourcedir &quot;',$dirname,'&quot;')"/>

	<xsl:if test="//user-widget">
		<xsl:value-of select="concat('&#xA;if {[file exists ',$file,'.mod]} {source ',$file,'.mod}')"/>
	</xsl:if>
	
	<xsl:value-of select="concat('&#xA;if {[file exists ',$file,'.sci]} {ScilabEval &quot;exec(\&quot;',$file,'.sci\&quot;,-1)&quot;}')"/>
	
	<xsl:apply-templates/>
	
	<xsl:text>&#xA;catch {wm state . withdrawn};</xsl:text>
	<xsl:text>&#xA;bind Table &lt;Return&gt; {runScilab {}; ::tk::table::MoveCell %W 1 0};</xsl:text>
	<xsl:text>&#xA;set keep yes</xsl:text>
	<xsl:text>&#xA;DynamicHelp::configure -bg white -font mainFont -delay 200</xsl:text>
</xsl:template>

<xsl:template match="toplevel">

	<xsl:variable name="top">
		<xsl:value-of select="concat('.',@label)"/>
	</xsl:variable>

	<xsl:value-of select="concat('&#xA;if [catch {toplevel ',$top,'}] {destroy ',$top,';toplevel ',$top,'};')"/>
			
	<xsl:apply-templates/>
	
	<xsl:call-template name="pack-children">
		<xsl:with-param name="window">
			<xsl:value-of select="$top"/>
		</xsl:with-param>
	</xsl:call-template>	

	<xsl:value-of select="concat('&#xA;wm resizable ',$top,' 0 0;')"/>
	<xsl:value-of select="concat('&#xA;wm title ',$top,' &quot;',@title,'&quot;')"/>

	<xsl:choose>
		<xsl:when test="ancestor::procedure[@type='tearoff']">
		</xsl:when>
		<xsl:when test="ancestor::procedure[@type='transient']">
			<!--<xsl:value-of select="concat('&#xA;wm transient ',$top,' .',ancestor::procedure/@parent)"/>-->
			<xsl:value-of select="concat('&#xA;set gc [grab current];if {$gc!=&quot;&quot;} {bind ',$top,' &lt;Destroy&gt; &quot;grab $gc&quot;}')"/>
			<xsl:value-of select="concat('&#xA;grab ',$top)"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="concat('&#xA;bind ',$top,' &lt;Destroy&gt; {set keep no}')"/>	
		</xsl:otherwise>
	</xsl:choose>

</xsl:template>

<xsl:template match="font-definition">
	<xsl:value-of select="concat('&#xA;catch {font create ',@name,' -family ',@family,
	                            ' -slant ',@slant,' -size ',@size,' -weight ',@weight,'}')"/>
</xsl:template>

<xsl:template name="name-or-id">
		<xsl:choose>
			<xsl:when test="@name">
				<xsl:value-of select="@name"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="generate-id()"/>
			</xsl:otherwise>
		</xsl:choose>	
</xsl:template>

<xsl:template name="pack-children">
	<xsl:param name="window"/>
	<xsl:variable name="top" select="concat('.',ancestor-or-self::toplevel[1]/@label)"/>
	<xsl:for-each select="*[not(self::procedure or self::command)]">
		<xsl:value-of select="concat('&#xA;pack ',$top,'.')"/>
		<xsl:call-template name="name-or-id"/>
		<xsl:value-of select="concat(' -in ',$window)"/>
		<xsl:if test="../@packside">
			<xsl:value-of select="concat(' -side ',../@packside)"/>
		</xsl:if>
		<xsl:if test="@anchor">
			<xsl:value-of select="concat(' -anchor ',@anchor)"/>
		</xsl:if>
		<xsl:if test="../@padx">
			<xsl:value-of select="concat(' -padx ',../@padx)"/>
		</xsl:if>
		<xsl:if test="../@pady">
			<xsl:value-of select="concat(' -pady ',../@pady)"/>
		</xsl:if>
		<xsl:if test="../@ipadx">
			<xsl:value-of select="concat(' -ipadx ',../@ipadx)"/>
		</xsl:if>
		<xsl:if test="../@ipady">
			<xsl:value-of select="concat(' -ipady ',../@ipady)"/>
		</xsl:if>
		<xsl:if test="@expand">
			<xsl:value-of select="concat(' -expand ',@expand)"/>
		</xsl:if>
		<xsl:if test="@fill">
			<xsl:value-of select="concat(' -fill ',@fill)"/>
		</xsl:if>
	</xsl:for-each>
</xsl:template>


<xsl:template match="frame[(not(@text)) or (@text='')]">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>
	<xsl:value-of select="concat('&#xA;frame ',$window)"/>
	<xsl:if test="@relief">
		<xsl:value-of select="concat(' -relief ',@relief,' -bd 2')"/>
	</xsl:if>
	<xsl:apply-templates/>
	<xsl:call-template name="pack-children">
		<xsl:with-param name="window">
			<xsl:value-of select="$window"/>
		</xsl:with-param>
	</xsl:call-template>
</xsl:template>

<xsl:template match="frame[@text and (@text!='')]">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>
	<xsl:value-of select="concat('&#xA;TitleFrame ',$window,' -ipad 0 -text &quot;',@text,'&quot;')"/>
	<xsl:if test="@relief">
		<xsl:value-of select="concat(' -relief ',@relief,' -bd 2')"/>
	</xsl:if>
	<xsl:if test="@font">
		<xsl:value-of select="concat(' -font ',@font)"/>
	</xsl:if>	
	<xsl:apply-templates/>
	<xsl:call-template name="pack-children">
		<xsl:with-param name="window">
			<xsl:value-of select="concat($window,'.f')"/>
		</xsl:with-param>
	</xsl:call-template>
</xsl:template>

<xsl:template match="saveTable">
	<xsl:variable name="table" select="concat('.',ancestor::toplevel[1]/@label,'.',@name)"/>
 	<xsl:value-of select="'set savefile [tk_getSaveFile -defaultextension .dat -filetypes {{{data files} {.dat}}} ];set fp [open $savefile w];puts $fp ['"/>
	<xsl:value-of select="concat($table,' selection set origin end; selection get];close $fp;')"/>
</xsl:template>

<xsl:template match="loadTable">
	<xsl:variable name="label" select="@name"/>
	<xsl:variable name="table" select="concat('.',ancestor::toplevel[1]/@label,'.',@name)"/>
	<xsl:variable name="rows" select="@rows"/>
	<xsl:variable name="cols" select="@cols"/>
	<xsl:value-of select="'set openfile [tk_getOpenFile -filetypes {{{data files} {.dat}}}]; set fp [open $openfile r];'"/>
	<xsl:value-of select="concat(' for {set i 1} {$i&lt;=',$rows+1,'} {incr i} {')"/>
	<xsl:value-of select="concat(' for {set j 1} {$j&lt;=',$cols+1,'} {incr j} {array unset ',$label,' $i,$j}};')"/>
	<xsl:value-of select="' clipboard clear;while {![eof $fp]} {clipboard append [gets $fp]};close $fp;'"/>
	<xsl:value-of select="concat($table,' activate origin; event generate ',$table,' &lt;&lt;Paste&gt;&gt;')"/>
	<xsl:text>&#xA;runScilab {}</xsl:text>
</xsl:template>

<xsl:template match="eraseTable">
	<xsl:call-template name="erasetable"/>
	<xsl:text>&#xA;runScilab {}</xsl:text>
</xsl:template>

<xsl:template name="erasetable">
	<xsl:variable name="label" select="@name"/>
	<xsl:variable name="rows" select="@rows"/>
	<xsl:variable name="cols" select="@cols"/>
  	<xsl:value-of select="concat('&#xA;for {set i 1} {$i&lt;=',$rows+1,'} {incr i} {')"/>
	<xsl:value-of select="concat(' for {set j 1} {$j&lt;=',$cols+1,'} {incr j} {')"/>
	<xsl:value-of select="concat('array unset ',$label,' $i,$j}};')"/>
</xsl:template>


<xsl:template match="table">
	<xsl:variable name="top" select="concat('.',ancestor::toplevel[1]/@label)"/>
	<xsl:variable name="window" select="concat($top,'.',@name)"/>
	<xsl:variable name="cellzero" select="concat($window,'.f')"/>
	<xsl:value-of select="concat('&#xA;unset -nocomplain ',@name)"/>
	<xsl:value-of select="concat('&#xA;table ',$window)"/>
	<xsl:text> -cache 1 </xsl:text>
	<xsl:text> -maxwidth 400 </xsl:text>
	<xsl:text> -maxheight 200 </xsl:text>
	<xsl:text> -rowseparator ";" </xsl:text>
	<xsl:text> -colseparator "," </xsl:text>
	<xsl:text> -multiline 0 </xsl:text>
	<xsl:text> -selectmode extended </xsl:text>	
	<xsl:text> -resizeborders none </xsl:text>	
	<xsl:text> -titlecols 1 </xsl:text>
	<xsl:text> -titlerows 1 </xsl:text>
	<xsl:value-of select="concat(' -state ',@state,' -rows ',@rows+1, ' -cols ',@cols+1,' -variable ',@name,'&#xA;')"/>
		
	 <xsl:value-of select="$window"/>
     <xsl:text> width 0 11;&#xA;</xsl:text>
	
	<xsl:choose>
		<xsl:when test="@colheaders > 0">	
		    <xsl:value-of select="$window"/>
		    <xsl:text> height 0 2;&#xA;</xsl:text>
		    <xsl:text>for {set i 1} {$i&lt;=</xsl:text>
		    <xsl:value-of select="@rows"/>
		    <xsl:text>} {incr i} {</xsl:text>
		    <xsl:value-of select="$window"/>
		    <xsl:text> set $i,0 $i};</xsl:text>
			<xsl:value-of select="concat('&#xA;',$window,' tag configure title -relief groove;&#xA;')"/>	
		</xsl:when>
		<xsl:when test="@rowheaders > 0">
		    <xsl:value-of select="$window"/>
		    <xsl:text> width 0 20;&#xA;</xsl:text>
		    <xsl:value-of select="$window"/>
		    <xsl:text> height 0 2;&#xA;</xsl:text>
		    <xsl:text>for {set i 1} {$i&lt;=</xsl:text>
		    <xsl:value-of select="@cols"/>
		    <xsl:text>} {incr i} {</xsl:text>
		    <xsl:value-of select="$window"/>
		    <xsl:text> set 0,$i $i}</xsl:text>
		    <xsl:text>;&#xA;</xsl:text>
		    <xsl:value-of select="$window"/>
			<xsl:text> tag configure title -relief groove;&#xA;</xsl:text>
		</xsl:when>
		<xsl:otherwise>
		    <xsl:value-of select="$window"/>
			<xsl:text> height 0 0;&#xA;</xsl:text>	
		    <xsl:value-of select="$window"/>
			<xsl:text> width 0 0;&#xA;</xsl:text>
		    <xsl:value-of select="$window"/>
			<xsl:text> tag configure title -background #d9d9d9;&#xA;</xsl:text>
		</xsl:otherwise>
	</xsl:choose>

	<xsl:apply-templates/>

	<xsl:choose>
    	<xsl:when test="buttonbox">
	 		<xsl:value-of select="concat($window,' window configure 0,0 -window ',$top,'.',generate-id(buttonbox),'&#xA;')"/>
        </xsl:when>
        <xsl:otherwise>	
	 		<xsl:value-of select="concat('frame ',$cellzero,'&#xA;')"/>
	 		<xsl:value-of select="concat($window,' window configure 0,0 -window ',$cellzero,'&#xA;')"/>
		</xsl:otherwise>
    </xsl:choose>
    
</xsl:template>


<xsl:template match="buttonbox">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>	
	<xsl:value-of select="concat('&#xA;ButtonBox ',$window,' -spacing ',@spacing,' -padx 1 -pady 1 &#xA;')"/>
    	
	<xsl:apply-templates/>

</xsl:template>


<xsl:template match="button[not(@image)]">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>	
	<xsl:value-of select="concat('&#xA;button ',$window,
							' -font mainFont -relief groove -text {',text,
							'} -command {')"/>
	<xsl:apply-templates select="command"/>
	<xsl:text>}&#xA;</xsl:text>
</xsl:template>

<xsl:template match="button[@image]">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>	
	<xsl:value-of select="concat('&#xA;Button ',$window,
							' -font mainFont -relief flat -bd 0 -image ',@image,
							' -command {')"/>
	<xsl:apply-templates select="command"/>
	<xsl:text>}</xsl:text>	
	<xsl:if test="helptext">
		<xsl:value-of select="concat(' -helptext &quot;',helptext,'&quot;')"/>
	</xsl:if>
	<xsl:text>&#xA;</xsl:text>	
</xsl:template>

<xsl:template match="buttonbox/button[@image]">
	<xsl:variable name="buttonbox" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id(..))"/>	
	<xsl:value-of select="concat($buttonbox,' add ',
							' -font mainFont -relief flat -bd 0 -image ',@image,
							' -command {')"/>
	<xsl:apply-templates select="command"/>
	<xsl:text>}</xsl:text>	
	<xsl:if test="helptext">
		<xsl:value-of select="concat(' -helptext &quot;',helptext,'&quot;')"/>
	</xsl:if>
	<xsl:text>&#xA;</xsl:text>	
</xsl:template>

<xsl:template match="label">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>	
	<xsl:value-of select="concat('&#xA;Label ',$window)"/> 
	<xsl:choose>
		<xsl:when test="@font">
			<xsl:value-of select="concat(' -font ',@font)"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:text> -font mainFont </xsl:text>
		</xsl:otherwise>
	</xsl:choose>	
    <xsl:if test="@width">
    	<xsl:value-of select="concat(' -wraplength ',@width)"/>
    </xsl:if>
    <xsl:if test="@justify">
    	<xsl:value-of select="concat(' -justify ',@justify)"/>
    </xsl:if>	
	<xsl:value-of select="concat(' -text {',text,'}')"/>	
	<xsl:if test="helptext">
		<xsl:value-of select="concat(' -helptext {',helptext,'}')"/>	
	</xsl:if>
</xsl:template>

<xsl:template match="entry">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>	
	<xsl:value-of select="concat('&#xA;LabelEntry ',$window,' -font mainFont -entrybg white -disabledforeground black',
							' -textvariable ',@variable,' -width ',@width,' -state ',@state)"/> 
	<xsl:if test="@text"><xsl:value-of select="concat(' -labelfont mainFont -label ','&quot;',@text,'&quot;')"/></xsl:if>
	<xsl:if test="command">	
		<xsl:value-of select="concat(' -command {',command,'}')"/>
	</xsl:if>    
	<xsl:value-of select="concat('&#xA;global ',@variable)"/>
    <xsl:if test="value">
        <xsl:value-of select="concat('&#xA;set ',@variable, ' ',value)"/>
    </xsl:if>

</xsl:template>

<xsl:template match="scale">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>	
	<xsl:value-of select="concat('&#xA;scale ',$window,' -font mainFont -variable ',@variable,
								' -width ',@width,' -state ',@state,' -from ',@from,' -to ',@to,
								' -resolution ',@resolution,' -bigincrement ',@resolution*10,
								' -orient horizontal -width ',@width)"/>
	<xsl:if test="command">
		<xsl:value-of select="concat(' -command {',command,'}')"/>
	</xsl:if>
	<xsl:if test="@repeatdelay">
		<xsl:value-of select="concat(' -repeatdelay ',@repeatdelay,' -repeatinterval 10')"/>
	</xsl:if>

	<xsl:value-of select="concat('&#xA;bind ',$window,' &lt;ButtonRelease-1&gt; {set but release;runScilab {}}')"/>
	<xsl:value-of select="concat('&#xA;bind ',$window,' &lt;ButtonPress-1&gt; {set but press}')"/>

</xsl:template>

<xsl:template match="jog">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>	
	<xsl:value-of select="concat('&#xA;jog {',$window,'} {',@variable,'} {',@from,'} {',@to,'} {',@resolution,'} {',command,'} {',@period,'}')"/>
</xsl:template>



<xsl:template match="litteral">
	<xsl:value-of select="concat('&#xA;',.)"/>
</xsl:template>

<xsl:template match="command">
	<xsl:apply-templates/>
</xsl:template>

<xsl:template match="set">
	<xsl:value-of select="concat('&#xA;set ',@variable,' {',@value,'}')"/>
</xsl:template>

<xsl:template match="unset">
	<xsl:value-of select="concat('&#xA;unset -nocomplain ',@variable)"/>
</xsl:template>

<xsl:template match="unsetmatrix">
	<xsl:call-template name="erasetable"/>
</xsl:template>


<xsl:template match="setmatrix">
	<xsl:if test="not(@mode) or (@mode!='init') or not(ancestor::procedure)">
	    <xsl:apply-templates/>
    </xsl:if>
</xsl:template>


<xsl:template match="setmatrix/row">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',../@name)"/>
	<xsl:value-of select="concat('&#xA;',$window,' set row ',@number,',',@from,' {')"/>
	<xsl:apply-templates/>
    <xsl:value-of select="'}'"/>
</xsl:template>

<xsl:template match="setmatrix//value">
	<xsl:value-of select="concat(' &quot;',.,'&quot;')"/>
</xsl:template>


<xsl:template match="setmatrix/col">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',../@name)"/>
	<xsl:value-of select="concat('&#xA;',$window,' set col ',@from,',',@number,' {')"/>
	<xsl:apply-templates/>
    <xsl:value-of select="'}'"/>
</xsl:template>

<xsl:template match="table/row">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',../@name)"/>
	<xsl:if test="not(ancestor::procedure)">
		<xsl:value-of select="concat($window,' set row ',position(),',0 {&quot;',name,'&quot; ',translate(value,',',' '),'};&#xA;')"/>
	</xsl:if>
</xsl:template>

<xsl:template match="table/col">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',../@name)"/>
	<xsl:if test="not(ancestor::procedure)">
		<xsl:value-of select="concat($window,' set col 0,',position(),' {&quot;',name,'&quot; ',translate(value,',',' '),'};&#xA;')"/>
	</xsl:if>
	<xsl:if test="name">
		<xsl:value-of select="concat($window,' width ',position(),' 20;&#xA;')"/>
	</xsl:if>
</xsl:template>


<xsl:template match="PagesManager[@style='notebook']">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>	
	<xsl:value-of select="concat('&#xA;NoteBook ',$window,' -font ',@font,' -tabpady {5 10} ')"/>

	<xsl:apply-templates/>

	<xsl:value-of select="concat('&#xA;',$window,' compute_size')"/>
    <xsl:choose>
    	<xsl:when test="@raise">
        		<xsl:value-of select="concat('&#xA;',$window,' raise ',@raise)"/>
		</xsl:when>
		<xsl:otherwise>
	    	<xsl:value-of select="concat('&#xA;',$window,' raise ',page[1]/@name)"/>
		</xsl:otherwise>
    </xsl:choose>
</xsl:template>


<xsl:template match="PagesManager[@style='plain']">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>	
	<xsl:value-of select="concat('&#xA;PagesManager ',$window)"/>

	<xsl:apply-templates/>

	<xsl:value-of select="concat('&#xA;',$window,' compute_size')"/>
    <xsl:choose>
    	<xsl:when test="@raise">
        		<xsl:value-of select="concat('&#xA;',$window,' raise ',@raise)"/>
		</xsl:when>
		<xsl:otherwise>
	    	<xsl:value-of select="concat('&#xA;',$window,' raise ',page[1]/@name)"/>
		</xsl:otherwise>
    </xsl:choose>
</xsl:template>


<xsl:template match="PagesManager[@style='notebook']/page">
	<xsl:variable name="manager" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id(..))"/>	
	<xsl:value-of select="concat('&#xA;',$manager,' insert ',position(),' ',@name,' -text &quot;',@text,'&quot;')"/>

	<xsl:apply-templates/>

	<xsl:call-template name="pack-children">
		<xsl:with-param name="window">
			<xsl:value-of select="concat($manager,'.f',@name)"/>
		</xsl:with-param>
	</xsl:call-template>

</xsl:template>


<xsl:template match="PagesManager[@style='plain']/page">
	<xsl:variable name="manager" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id(..))"/>
	<xsl:value-of select="concat('&#xA;',$manager,' add ',@name)"/>

	<xsl:apply-templates/>

	<xsl:call-template name="pack-children">
		<xsl:with-param name="window">
			<xsl:value-of select="concat($manager,'.f',@name)"/>
		</xsl:with-param>
	</xsl:call-template>

</xsl:template>

<xsl:template match="UserFrame">

	<xsl:apply-templates/>

	<xsl:call-template name="pack-children">
		<xsl:with-param name="window">
			<xsl:value-of select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id(..),'.frame')"/>
		</xsl:with-param>
	</xsl:call-template>

</xsl:template>

<xsl:template match="MainFrame">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>

	<xsl:apply-templates select="MenuBar"/>

	<xsl:value-of select="concat('&#xA;MainFrame ',$window,' -menu $',generate-id(),'menu',' -textvariable status -separator both')"/>
	<xsl:if test="@menubarfont">
		<xsl:value-of select="concat(' -menubarfont ',@menubarfont)"/>
	</xsl:if>
	<xsl:if test="@menuentryfont">
		<xsl:value-of select="concat(' -menuentryfont ',@menuentryfont)"/>
    </xsl:if> 
	<xsl:if test="@statusbarfont">
		<xsl:value-of select="concat(' -statusbarfont ',@statusbarfont)"/>
    </xsl:if>
	
	<xsl:apply-templates select="ToolBar|UserFrame"/>
</xsl:template>

<xsl:template match="MenuBar">
	<xsl:value-of select="concat('&#xA;set ',generate-id(..),'menu ','{')"/>

	<xsl:apply-templates/>

	<xsl:text>}</xsl:text>
</xsl:template>

<xsl:template match="MenuBar/Menu">
	<xsl:value-of select="concat('{',name,'} {} {} 0 {')"/>

	<xsl:apply-templates select="item"/>

	<xsl:text>} </xsl:text>
</xsl:template>

<xsl:template match="ToolBar"/>

<xsl:template match="MenuBar/Menu/item">
	<xsl:value-of select="concat('{ ',@type,' {',text,'} {} {',description,'} {} -command ')"/>
	<xsl:apply-templates select="command"/>
	<xsl:text> } </xsl:text>
</xsl:template>

<xsl:template match="MenuBar/Menu/item/command">
	<xsl:text>{</xsl:text>
	<xsl:apply-templates/>
	<xsl:text>}</xsl:text>
</xsl:template>

<xsl:template match="raise">
	<xsl:variable name="top" select="concat('.',ancestor::toplevel[1]/@label,'.')"/>
	<xsl:variable name="manager" select="generate-id(ancestor::MainFrame[1]//PagesManager)"/>
	<xsl:value-of select="concat($top,$manager,' raise ',@name)"/>
</xsl:template>

<xsl:template match="image">
	<xsl:variable name="image" select="concat('.',ancestor::toplevel[1]/@label,'.image')"/>
	<xsl:variable name="label" select="concat('.',ancestor::toplevel[1]/@label,'.',generate-id())"/>
    <xsl:value-of select="concat('&#xA;if {[catch {image create photo ',$image,' -file ',@href,'}]} {')"/>
	<xsl:value-of select="concat('image create photo ',$image,' -file [file join $xmllab noarch images notfound.gif]}')"/>
	<xsl:value-of select="concat('&#xA;label ',$label,' -image ',$image,' -relief groove')"/>
</xsl:template>

<xsl:template match="menubutton">
	<xsl:variable name="window" select="concat('.',ancestor::toplevel[1]/@label,'.',@name)"/>
	<xsl:value-of select="concat('&#xA;ComboBox ',$window,' -font mainFont -editable no ',
							' -insertontime 0 -entrybg white -width 32 ',
							' -textvariable ',@textvariable,' -values {')"/>
	<xsl:apply-templates select="menu"/>
	<xsl:value-of select="concat('&#xA;',$window,' setvalue first')"/>
</xsl:template>

<xsl:template match="menubutton/menu">
	<xsl:variable name="menubutton" select="concat('.',ancestor::toplevel[1]/@label,'.',../@name)"/>
	<xsl:apply-templates select="item"/>
	<xsl:value-of select="concat('} -modifycmd {switch [',$menubutton,' getvalue] {')"/> 
		<xsl:for-each select="item">
			<xsl:value-of select="concat(' ',position()-1,' { ')"/>
                <xsl:apply-templates select="command"/>
				<xsl:choose>
					<xsl:when test="@editable='yes'">
						<xsl:value-of select="concat('&#xA;',$menubutton,' configure -editable yes -insertontime 500')"/>
					</xsl:when>
					<xsl:otherwise>
						<xsl:value-of select="concat('&#xA;',$menubutton,' configure -editable no -insertontime 0')"/>
					</xsl:otherwise>
				</xsl:choose>
			<xsl:text>}</xsl:text>
		</xsl:for-each>
	<xsl:text>}}</xsl:text>
</xsl:template>

<xsl:template match="menubutton/menu/item">
	<xsl:value-of select="concat('&quot;',text,'&quot; ')"/>
</xsl:template>

<xsl:template match="scilab-script">
	<xsl:value-of select="concat('&#xA;ScilabEval {exec(&quot;',@href,'&quot;,-1)}')"/>
</xsl:template>

<xsl:template match="scilab-eval">
	<xsl:value-of select="concat('&#xA;ScilabEval {',@cmd,'}')"/>
</xsl:template>


<xsl:template match="comment">
	<xsl:value-of select="concat('&#xA;# ',normalize-space(.))"/>
</xsl:template>

<xsl:template match="user-widget">
	<xsl:variable name="top" select="concat('.',ancestor::toplevel[1]/@label,'.')"/>
	<xsl:value-of select="concat('&#xA;',@procedure,' ',$top,' ',@name,' create {',@text,'} {',@variables,'} {',@values,'} {',@parameters,'} {',@lang,'}')"/>
</xsl:template>


<xsl:template match="procedure">
	<xsl:value-of select="concat('&#xA;proc ',@name,' {',@parms,'} ')"/>
	<xsl:apply-templates select="body"/>
</xsl:template>

<xsl:template match="body">
	<xsl:text>{</xsl:text>
	<xsl:apply-templates/>
	<xsl:text>}&#xA;</xsl:text>
</xsl:template>

<xsl:template match="write-file">
	<xsl:value-of select="concat('&#xA;set fp [open [file join &quot;',@dir,'&quot; &quot;',@name,'&quot;] w];')"/>
	<xsl:text>&#xA;puts $fp "&lt;?xml version=\"1.0\" encoding=\"ISO-8859-1\"?&gt;"</xsl:text>	
	<xsl:text>&#xA;puts $fp "</xsl:text>
    <xsl:apply-templates mode="write-file"/>
    <xsl:text>"&#xA;close $fp;&#xA;</xsl:text>
</xsl:template>

<xsl:template match="xslt">
	<xsl:text>&#xA;exec xsltproc</xsl:text>
       <xsl:if test="@validate='no'">
    	<xsl:text> --novalid</xsl:text>
	</xsl:if>
    <xsl:value-of select="concat(' --stringparam default-system-uri &quot;',@system-uri,'&quot;',' --stringparam default-public-id &quot;',@public-id,'&quot;',' --stringparam workdir &quot;',@workdir,'&quot;',' --stringparam output &quot;',@output,'&quot; ',@stylesheet,' &quot;',@input,'&quot;')"/>
</xsl:template>


<xsl:template match="rename-file">
	 <xsl:value-of select="concat('&#xA;file rename -force &quot;',@old,'&quot; &quot;',@new,'&quot;')"/>
</xsl:template>


<xsl:template match="delete-file">
	 <xsl:value-of select="concat('&#xA;file delete -force [file join &quot;',@dir,'&quot; &quot;',@name,'&quot;]')"/>
</xsl:template>



<xsl:template match="*" mode="write-file">
	<xsl:value-of select="concat('&lt;',name())"/>
    <xsl:apply-templates select="@*" mode="write-file"/>
	<xsl:text>&gt;</xsl:text>
	<xsl:apply-templates select="*|text()" mode="write-file"/>
    <xsl:value-of select="concat('&lt;/',name(),'&gt;')"/>
</xsl:template>

<xsl:template match="@*" mode="write-file">
	<xsl:value-of select="concat(' ',name(),'=\&quot;',.,'\&quot;')"/>
</xsl:template>


</xsl:stylesheet>













