<?xml version="1.0" encoding="ISO-8859-1"?>

<xsl:stylesheet xmlns:exslt="http://exslt.org/common" 
                xmlns:str="http://exslt.org/strings" 
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
		version="1.0" 
		exclude-result-prefixes="exslt str">

  <xsl:output method="xml" indent="yes" encoding="ISO-8859-1"/>

  <xsl:key name="subsections" match="subsection|database" use="@label"/>
  <xsl:key name="matrices" match="matrix" use="@label"/>
  <xsl:key name="scalars" match="scalar" use="@label"/>

  <xsl:param name="file"/>
  <xsl:param name="outputdir"/>
  <xsl:param name="sourcename"/>
  <xsl:param name="dirnamename"/>
  <xsl:param name="os"/>
  <xsl:param name="language"/>
  <xsl:param name="toplevel">xmllab</xsl:param>
  <xsl:param name="screenshots"/>
  <xsl:param name="appname"/>
  <xsl:param name="havesource"/>

  <xsl:variable name="lang">
  	<xsl:choose>
    	<xsl:when test="$language='DEFAULT'">
        	<xsl:value-of select="/simulation/@default-lang"/>
        </xsl:when>
        <xsl:otherwise>
        	<xsl:value-of select="$language"/>
        </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>


<!--  <xsl:variable name="version">
    <xsl:value-of select="document('version.xml')/xmllab/@version"/>
  </xsl:variable>-->

  <xsl:include href="localization.xsl"/>

  <xsl:strip-space elements="*"/>
  
  <xsl:template match="/">
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="simulation">

    <tkml>

      <xsl:variable name="label">
        <xsl:choose>
          <xsl:when test="$appname!='NONAME'">
            <xsl:value-of select="$appname"/>
          </xsl:when>
          <xsl:when test="@label">
            <xsl:value-of select="@label"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$file"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:choose>
        <xsl:when test="($os='Linux') or ($os='SunOs') or ($os='Darwin')">
          <xsl:value-of select="concat('&#10;tk appname ',$label)"/>
        </xsl:when>
        <xsl:when test="$os='Windows NT'">
          <xsl:value-of select="concat('&#10;package require dde;&#10;dde servername ',$label)"/>
        </xsl:when>
      </xsl:choose>

      <command>
        <litteral>package require -exact Tktable 2.8</litteral>
      </command>

      <xsl:copy-of select="exslt:node-set($strings)/credits[@lang=$lang]"/>

      <command>
        <set variable="toplevelWindow" value="{concat('.',$toplevel)}"/>
      </command>

      <xsl:apply-templates select="parameters/section[@hidden='yes']" mode="pagemanager"/>
      <xsl:apply-templates select="parameters/section/subsection[@hidden='yes']"/>

      <toplevel label="{$toplevel}">
		<xsl:attribute name="title">
			<xsl:if test="not(header/title[@lang=$lang])">
				<xsl:message>
                	<xsl:value-of select="concat('Warning, no title for language &quot;',$lang,'&quot;') "/>
                	<xsl:value-of select="concat(' (default language is &quot;',/simulation/@default-lang, '&quot;)')"/>
                </xsl:message> 
            </xsl:if>
        	<xsl:value-of select="normalize-space(header/title[@lang=$lang])"/>
            <xsl:if test="$screenshots='yes'">
	        	<xsl:value-of select="' (0)'"/>
			</xsl:if>
        </xsl:attribute>
       
        <MainFrame menubarfont="menuFont" menuentryfont="menuFont" statusbarfont="menuFont">

          <MenuBar>

            <Menu>  <!-- This is the "File" menu -->
              <name>
                <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@file"/>
              </name>
              <item type="command">
                <text>
                  <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@run"/>
                </text>
                <description>
                  <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@rundesc"/>
                </description>
                <command>
                  <litteral>runScilab {}</litteral>
                </command>
              </item>
              <item type="command">
                <text>
                  <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@savesession"/>
                </text>
                <description>
                  <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@savesessiondesc"/>
                </description>
                <command>
                  <xsl:text>set savefile [tk_getSaveFile -initialdir $SESSIONDIR];set fp [open $savefile w];</xsl:text>
                  <xsl:call-template name="generate-save-cmd"/>
                </command>
              </item>
              <item type="command">
                <text>
                  <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@loadsession"/>
                </text>
                <description>
                  <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@loadsessiondesc"/>
                </description>
                <command>
                  <xsl:text>set sessionFile [tk_getOpenFile -initialdir $SESSIONDIR];</xsl:text>
                  <xsl:text>source $sessionFile;</xsl:text>
                  <xsl:text>runScilab {}</xsl:text>
                </command>
              </item>
              <xsl:if test="//parameters/@widget='plain'">
                <xsl:apply-templates select="save" mode="menu"/>
              </xsl:if>
              <item type="command">
                <text>
                  <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@quit"/>
                </text>
                <description>
                  <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@quitdesc"/>
                </description>
                <command>set keep no</command>
              </item>
            </Menu>

            <xsl:apply-templates select="actions" mode="menu"/>  <!-- This is the "Actions" menu -->
	    
            <xsl:if test="parameters/@widget='plain'">
              
              <xsl:if test="(parameters//point[@widget!='hidden']) or (parameters//scalar[@widget!='hidden']) or (parameters//matrix[@widget!='hidden'])">
	              <Menu>  <!-- This is the "Parameters" menu -->
                    <name>
                      <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@parameters"/>
                    </name>
	 	               <xsl:apply-templates select="parameters/section" mode="menu"/>                
                  </Menu>
	          </xsl:if>
	      
              <Menu>  <!-- This is the "XMLlab" menu -->
                <name>XMLlab</name>
                <item type="command">
                  <text>
                    <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@about"/>
                  </text>
                  <description>
                    <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@aboutdesc"/>
                  </description>
                  <command>
                    <raise name="credits"/>
                  </command>
                </item>
                <item type="command">
                  <text>
                    <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@abouthis"/>
                  </text>
                  <description>
                    <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@aboutthisdesc"/>
                  </description>
                  <command>
                    <raise name="notes"/>
                  </command>
                </item>
              </Menu>


            </xsl:if>

              <xsl:if test="$havesource='yes'">
	      <Menu> <!-- This is the "Languages" menu -->
              	<name><xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@languages"/></name>
                <xsl:for-each select="header/title">
					<item type="command">
                    	<text>
                        	<xsl:value-of select="exslt:node-set($strings)/strings[@lang=current()/@lang]/@langlang"/>
                        </text>
                        <command>
							<xsl:value-of select="concat('xmllab 4 &quot;-lang ',@lang,' -run \&quot;',$sourcename,'\&quot;&quot;')"/>
                        </command>
                    </item>
                </xsl:for-each> 
              </Menu>
	     </xsl:if>


          </MenuBar>

          <ToolBar/>

          <UserFrame padx="5" pady="5">
          
	    <PagesManager style="{parameters/@widget}" font="mainFont">
          <xsl:if test="not ((parameters//point[@widget!='hidden']) or (parameters//scalar[@widget!='hidden']) or (parameters//matrix[@widget!='hidden']))">
			<xsl:attribute name="raise">notes</xsl:attribute>
          </xsl:if>
              
	      <xsl:apply-templates select="parameters/section[@hidden='no']" mode="pagemanager"/>
              <xsl:apply-templates select="save" mode="pagemanager"/>
              
	      <page name="credits" text="XMLlab">
                <xsl:copy-of select="exslt:node-set($strings)/menucredits[@lang=$lang]/frame"/>
              </page>
              
	      <page name="notes" text="Notes">
                <frame packside="top" anchor="n" pady="5">
                  <label font="boldFont">
                    <text>
                      <xsl:value-of select="normalize-space(/simulation/header/title[(@lang=$lang) or (not(@lang))])"/>
                    </text>
                  </label>
                  <label font="mainFont">
                    <text>
                      <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@author"/>
                      <xsl:text> : </xsl:text>
                      <xsl:value-of select="/simulation/header/author"/>
                      <xsl:text>, </xsl:text>
                      <xsl:value-of select="/simulation/header/date"/>
                    </text>
                  </label>
                  <xsl:for-each select="notes[@lang=$lang or (not(@lang))]/p">
                    <label font="mainFont" width="300" justify="left">
                      <text>
                        <xsl:value-of select="normalize-space(.)"/>
                      </text>
                    </label>
                  </xsl:for-each>
                </frame>
              </page>
            
	    </PagesManager>
          
	    <xsl:if test="header/image">
              <xsl:copy-of select="header/image"/>
            </xsl:if>
          
	  </UserFrame>
	  
        </MainFrame>
	
        <xsl:for-each select="parameters//database">
          <xsl:for-each select="record[1]">
            <xsl:call-template name="database-item-command"/>
          </xsl:for-each>
        </xsl:for-each>

      </toplevel>

      <xsl:for-each select="parameters//scalar[not(ancestor::database)] | parameters//point/x1 | parameters//point/x2">
        <command>
          <unset variable="{@label}"/>
          <set variable="{@label}" value="{translate(normalize-space(value),' ','')}"/>
        </command>
      </xsl:for-each>

      <xsl:for-each select="//user-widget[@values and not(ancestor::database)]">
        <command>
          <xsl:variable name="values" select="@values"/>
          <xsl:for-each select="str:split(@variables)">
            <xsl:variable name="pos" select="position()"/>
            <unset variable="{.}"/>
            <set variable="{.}" value="{str:split($values)[$pos]}"/>
          </xsl:for-each>
        </command>
      </xsl:for-each>

      <command>
        <xsl:variable name="savecmd">
          <xsl:call-template name="generate-save-cmd"/>
        </xsl:variable>
        <set variable="save_session_cmd" value="{$savecmd}"/>
      </command>

    </tkml>

  </xsl:template>
  
  <xsl:template name="generate-save-cmd">

    <xsl:for-each select="//scalar|//point/x1 | //point/x2 | //database">
      <xsl:text>puts $fp "set </xsl:text>
      <xsl:value-of select="@label"/>
      <xsl:text> {$</xsl:text>
      <xsl:value-of select="@label"/>
      <xsl:text>}";</xsl:text>
    </xsl:for-each>

    <xsl:for-each select="//matrix">
      <xsl:text>puts $fp "array set </xsl:text>
      <xsl:value-of select="@label"/>
      <xsl:text> {[array get </xsl:text>
      <xsl:value-of select="@label"/>
      <xsl:text>]}";</xsl:text>
    </xsl:for-each>

    <xsl:for-each select="//user-widget[@save='yes']">

      <xsl:variable name="name">
        <xsl:choose>
          <xsl:when test="@label">
            <xsl:value-of select="@label"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="concat('u',generate-id())"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <xsl:value-of select="concat('set cmd [',@procedure,' ',$toplevel,' ',$name,' save {} {',@variables,'} {} {} {} ];')"/>

      <xsl:text>puts $fp $cmd;</xsl:text>

      <xsl:variable name="values" select="@values"/>

      <xsl:for-each select="str:split(@variables)">
        <xsl:variable name="pos" select="position()"/>
        <xsl:value-of select="concat('puts $fp &quot;set ',.,' {$',.,'}&quot;;')"/>
      </xsl:for-each>

    </xsl:for-each>

    <xsl:text>close $fp;</xsl:text>

  </xsl:template>
  
  <xsl:template match="parameters/section[@hidden='no']" mode="menu">

    <item type="command">
      <text>
        <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
      </text>
      <description>
        <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
      </description>
      <command>
        <raise name="{translate(generate-id(),'.','_')}"/>
      </command>
    </item>

  </xsl:template>
  
  <xsl:template match="parameters/section[@hidden='yes']" mode="menu"/>
  
  <xsl:template match="parameters/section[@hidden='no']" mode="pagemanager">

    <page name="{translate(generate-id(),'.','_')}" text="{normalize-space(title[(@lang=$lang) or (not(@lang))])}" pady="4">
      <xsl:if test="../@widget='plain'">
        <frame packside="top" anchor="n" pady="5">
          <label font="boldFont">
            <text>
              <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
            </text>
            <xsl:call-template name="helptext"/>
          </label>
        </frame>
      </xsl:if>
      <xsl:choose>
        <xsl:when test="frame">
          <xsl:apply-templates select="frame"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:apply-templates select="*[not(self::subsection[@hidden='yes'])]"/>
        </xsl:otherwise>
      </xsl:choose>
    </page>

  </xsl:template>
  
  <xsl:template match="parameters/section[@hidden='yes']" mode="pagemanager">

    <procedure name="{@label}" parms="top" parent="{$toplevel}" type="transient">
      <body>
        <toplevel label="{@label}" title="{normalize-space(title[(@lang=$lang) or (not(@lang))])}">
          <xsl:choose>
            <xsl:when test="frame">
              <xsl:apply-templates select="frame"/>
            </xsl:when>
            <xsl:otherwise>
              <frame packside="top" fill="both" expand="true" padx="5" pady="5">
                <xsl:apply-templates select="*[not(self::subsection[@hidden='yes'])]"/>
              </frame>
            </xsl:otherwise>
          </xsl:choose>
        </toplevel>
      </body>
    </procedure>

  </xsl:template>
  
  <xsl:template match="subsection[@hidden='no']">

    <xsl:variable name="label">
      <xsl:choose>
        <xsl:when test="@label">
          <xsl:value-of select="@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat('database',generate-id())"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <xsl:variable name="parent">
      <xsl:choose>
        <xsl:when test="parent::section[@hidden='yes']">
          <xsl:value-of select="parent::section/@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$toplevel"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <frame packside="top" pady="3" fill="both" expand="true">
      <xsl:if test="title">
        <xsl:attribute name="text">
          <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
        </xsl:attribute>
        <xsl:attribute name="font">boldFont</xsl:attribute>
      </xsl:if>
      <xsl:if test="@tearoff='yes'">
        <frame packside="right" fill="x" padx="7">
          <button anchor="ne" image="tearoffImage">
            <command>
              <xsl:value-of select="concat($label,' {',$parent,'}&#10;')"/>
            </command>
          </button>
        </frame>
      </xsl:if>
      <xsl:apply-templates/>
    </frame>

    <xsl:if test="@tearoff='yes'">
      <procedure name="{$label}" parms="top" parent="{$parent}" type="tearoff">
        <body>
          <xsl:for-each select="descendant::matrix">
            <command>
              <litteral>
                <xsl:value-of select="concat('.',$parent,'.',@label,' activate 0,0&#10;')"/>
              </litteral>
            </command>
          </xsl:for-each>
          <toplevel label="{$label}" title="{normalize-space(title[(@lang=$lang) or (not(@lang))])}">
            <frame packside="top" fill="both" expand="true" padx="5" pady="5">
              <xsl:apply-templates/>
            </frame>
          </toplevel>
        </body>
      </procedure>
    </xsl:if>

  </xsl:template>
  
  <xsl:template match="subsection[@hidden='yes']">

    <xsl:variable name="label">
      <xsl:choose>
        <xsl:when test="@label">
          <xsl:value-of select="@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat('database',generate-id())"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <procedure name="{$label}" parms="top" type="transient">
      <xsl:attribute name="parent">
        <xsl:choose>
          <xsl:when test="parent::section[@hidden='yes']">
            <xsl:value-of select="parent::section/@label"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="$toplevel"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <body>
        <toplevel label="{$label}" title="{normalize-space(title[(@lang=$lang) or (not(@lang))])}">
          <frame packside="top" fill="both" expand="true" padx="5" pady="5">
            <xsl:apply-templates/>
          </frame>
        </toplevel>
      </body>
    </procedure>

  </xsl:template>
  
  <xsl:template match="frame">
    <frame packside="{@packside}" fill="{@fill}" expand="{@expand}" padx="{@padx}" pady="{@pady}">
      <xsl:apply-templates/>
    </frame>
  </xsl:template>
  
  <xsl:template match="pack">
    <xsl:for-each select="str:split(@what)">
      <xsl:apply-templates select="key('subsections',.)"/>
    </xsl:for-each>
  </xsl:template>
  
  <xsl:template match="actions" mode="menu">
    <Menu>
      <name>
        <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
      </name>
      <xsl:apply-templates/>
    </Menu>
  </xsl:template>
  
  <xsl:template match="action">
 
    <item type="command">
      <text>
        <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
      </text>
      <description>
        <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
      </description>
      <command>
        <xsl:choose>
          <xsl:when test="script/text()">
            <xsl:element name="scilab-eval">
              <xsl:attribute name="cmd">
                <xsl:for-each select="str:split(script/text(),'&#10;')">
                  <xsl:value-of select="normalize-space(.)"/>
                  <xsl:if test="position()&lt;last()">
                    <xsl:text>,</xsl:text>
                  </xsl:if>
                </xsl:for-each>
				<xsl:call-template name="split-action-update">
                	<xsl:with-param name="content" select="@update"/>
                </xsl:call-template>
                <xsl:text>,exec(xmllab_run,-1)</xsl:text>
              </xsl:attribute>
            </xsl:element>
          </xsl:when>
          <xsl:otherwise>
            <xsl:element name="scilab-eval">
              <xsl:attribute name="cmd">
                <xsl:value-of select="concat('exec(&quot;',./script/@href,'&quot;,-1)')"/>
				    <xsl:call-template name="split-action-update">
                	    <xsl:with-param name="content" select="@update"/>
                    </xsl:call-template>
                <xsl:text>,exec(xmllab_run,-1)</xsl:text>
              </xsl:attribute>
            </xsl:element>
          </xsl:otherwise>
        </xsl:choose>
      </command>
    </item>
  
  </xsl:template>

<xsl:template name="action-update">
    <xsl:param name="label"/>
    <xsl:for-each select="key('scalars',$label)">
        <xsl:value-of select="concat(',TCL_SetVar(&quot;',@label,'&quot;,string(',@label,'))')"/>
      </xsl:for-each>
      <xsl:for-each select="key('matrices',$label)">
        <xsl:value-of select="concat(',TCL_SetMatrix(&quot;',@label,'&quot;,',@label,')')"/>
      </xsl:for-each>
</xsl:template>

<xsl:template name="split-action-update">
  <xsl:param name="content"/>
  <xsl:choose>
    <xsl:when test="contains($content, ' ')">
      <xsl:call-template name="action-update">
      	<xsl:with-param name="label" select="substring-before($content, ' ')"/>
      </xsl:call-template>
      <xsl:call-template name="split-action-update">
        <xsl:with-param name="content" select="substring-after($content, ' ')"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
     <xsl:call-template name="action-update">
      	<xsl:with-param name="label" select="$content"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

  
  
  <xsl:template name="database-item-command">
  
    <command>
      <xsl:for-each select="../scalar|../group/scalar">
        <set variable="{@label}" value="{value}"/>
      </xsl:for-each>
      <xsl:for-each select="scalar-value">
        <set variable="{@ref}" value="{.}"/>
      </xsl:for-each>
      <xsl:for-each select="../matrix|../group/matrix">
        <unsetmatrix name="{@label}" cols="{@cols}" rows="{@rows}"/>
        <xsl:call-template name="setmatrix"/>
      </xsl:for-each>
      <xsl:for-each select="matrix-value">
        <xsl:call-template name="setmatrix">
          <xsl:with-param name="label" select="@ref"/>
        </xsl:call-template>
      </xsl:for-each>
    </command>
  
  </xsl:template>
  
  <xsl:template name="database-menu">
  
    <frame packside="left" padx="5" anchor="w" fill="x">
  
      <xsl:variable name="label">
        <xsl:choose>
          <xsl:when test="@label">
            <xsl:value-of select="@label"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="concat('database',generate-id())"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>
  
      <menubutton textvariable="{$label}" variable="{concat($label,'index')}" name="{$label}">
        <text>truc</text>
        <menu>
          <xsl:for-each select="record">
            <item editable="no">
              <text>
                <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
              </text>
              <xsl:call-template name="database-item-command"/>
              <command>
                <litteral>runScilab {}</litteral>
              </command>
            </item>
          </xsl:for-each>
          <xsl:if test="@editable='yes'">
            <item editable="yes">
              <text>
                <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@userdefined"/>
              </text>
              <command/>
            </item>
          </xsl:if>
        </menu>
      </menubutton>
  
      <xsl:if test="@editable='yes' and $havesource='yes'">

        <button image="saveImage">
          <helptext>
            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@store"/>
          </helptext>
          <command>
            <write-file dir="{$dirname}" name="modify.xml" content="xml">
              <modify action="add" database="{$label}" record="${$label}" lang="{$lang}">
                <xsl:for-each select="scalar|group/scalar">
                  <scalar-value ref="{@label}">
                    <xsl:value-of select="concat('$',@label)"/>
                  </scalar-value>
                </xsl:for-each>
                <xsl:for-each select="matrix|group/matrix">
                  <matrix-value ref="{@label}">
                    <xsl:variable name="padd" select="str:padding(2*@rows,' ,')"/>
                    <xsl:variable name="reflabel" select="@label"/>
                    <xsl:variable name="cols" select="@cols"/>
                    <xsl:for-each select="str:split($padd,',')">
                      <row>
                        <value>
                          <xsl:value-of select="concat('[string map {\{\} &quot; &quot;} [.',$toplevel,'.',$reflabel,' get ',position(),',1 ',position(),',',$cols,']]')"/>
                        </value>
                      </row>
                    </xsl:for-each>
                  </matrix-value>
                </xsl:for-each>
                <xsl:for-each select="user-widget|group/user-widget">
                  <xsl:for-each select="str:split(@variables)">
                    <scalar-value ref="{.}">
                      <xsl:value-of select="concat('$',.)"/>
                    </scalar-value>
                  </xsl:for-each>
                </xsl:for-each>
              </modify>
            </write-file>
            <xsl:call-template name="database-xslt-process"/>
          </command>
        </button>
	
        <button image="deleteImage">
          <helptext>
            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@erase"/>
          </helptext>
          <command>
            <write-file dir="{$dirname}" name="modify.xml" content="xml">
              <modify action="delete" database="{$label}" record="${$label}"  lang="{$lang}"/>
            </write-file>
            <xsl:call-template name="database-xslt-process"/>
          </command>
        </button>
	
        <button image="homeImage">
          <helptext>
            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@default"/>
          </helptext>
          <command>
            <write-file dir="{$dirname}" name="modify.xml" content="xml">
              <modify action="makedefault" database="{$label}" record="${$label}"  lang="{$lang}"/>
            </write-file>
            <xsl:call-template name="database-xslt-process"/>
          </command>
        </button>
	
      </xsl:if>
      
    </frame>
    
  </xsl:template>
  
  <xsl:template name="database-xslt-process">
    <xslt workdir="{$outputdir}" validate="no" input="{$sourcename}" 
          output="{concat($sourcename,'.new')}" stylesheet="[file join $xmllab xsl modify.xsl]"
          public-id="{/simulation/@default-public-id}" system-uri="{/simulation/@default-system-uri}"/>
    <rename-file old="{concat($sourcename,'.new')}" new="{$sourcename}"/>
    <!--<delete-file dir="{$dirname}" name="modify.xml"/>-->
    <scilab-eval cmd="{concat('xmllab -lang ',$lang,' -run &quot;',$sourcename,'&quot;')}"/>
  </xsl:template>
  
  <xsl:template match="database">
  
    <xsl:variable name="parent">
      <xsl:choose>
        <xsl:when test="parent::section[@hidden='yes']">
          <xsl:value-of select="parent::section/@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="$toplevel"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
  
    <frame packside="top" anchor="n" pady="5" fill="both" expand="true">

      <xsl:choose>
        <xsl:when test="title">
          <xsl:attribute name="text">
            <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
          </xsl:attribute>
          <xsl:attribute name="font">boldFont</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:attribute name="relief">groove</xsl:attribute>
        </xsl:otherwise>
      </xsl:choose>

      <xsl:if test="@tearoff='yes'">
        <frame packside="right" fill="x" padx="7">
          <button anchor="ne" image="tearoffImage">
            <command>
              <xsl:value-of select="concat(@label,' {',$parent,'}&#10;')"/>
            </command>
          </button>
        </frame>
      </xsl:if>

      <xsl:call-template name="database-menu"/>
      <xsl:apply-templates select="scalar|group|matrix|user-widget"/>

    </frame>

    <xsl:if test="@tearoff='yes'">

      <procedure name="{@label}" parms="top" parent="{$parent}" type="tearoff">
        <body>
          <toplevel label="{@label}" title="{normalize-space(title[(@lang=$lang) or (not(@lang))])}">
            <frame packside="top" fill="both" expand="true" pady="5" padx="5">
              <xsl:call-template name="database-menu"/>
              <xsl:apply-templates select="scalar|group|matrix|user-widget"/>
            </frame>
          </toplevel>
        </body>
      </procedure>

    </xsl:if>

  </xsl:template>
  
  <xsl:template match="point[@widget='hidden']"/>
  
  <xsl:template match="point[@widget='entry']">

    <xsl:variable name="state">
      <xsl:choose>
        <xsl:when test="constraints|refconstraints|@state='disabled'">disabled</xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="@state"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <frame packside="left" padx="5" expand="true" fill="x">
      <label anchor="w" expand="true">
        <text>
          <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
        </text>
        <xsl:call-template name="helptext"/>
      </label>
      <entry anchor="e" variable="{x1/@label}" width="5" state="{$state}" text="{normalize-space(x1/name[(@lang=$lang) or (not(@lang))])}">
        <value>
          <xsl:value-of select="x1/value"/>
        </value>
        <command>runScilab {}</command>
      </entry>
      <entry anchor="e" variable="{x2/@label}" width="5" state="{$state}" text="{normalize-space(x2/name[(@lang=$lang) or (not(@lang))])}">
        <value>
          <xsl:value-of select="x2/value"/>
        </value>
        <command>runScilab {}</command>
      </entry>
    </frame>

  </xsl:template>
  
  
  <xsl:template name="helptext">
    <xsl:if test="notes">
      <helptext>
        <xsl:for-each select="notes[(@lang=$lang) or (not(@lang))]/p">
          <xsl:value-of select="normalize-space(.)"/>
          <xsl:if test="position()&lt;last()">
            <xsl:text/>
          </xsl:if>
        </xsl:for-each>
      </helptext>
    </xsl:if>
  
  </xsl:template>
  
  <xsl:template match="group">
  
    <xsl:if test="title">
      <frame packside="left" padx="5" expand="true" fill="x">
        <label anchor="w" expand="true" font="boldFont">
          <text>
            <xsl:value-of select="normalize-space(title[(@lang=$lang) or (not(@lang))])"/>
          </text>
        </label>
      </frame>
    </xsl:if>
    <frame packside="left" padx="2" anchor="e">
      <xsl:apply-templates/>
    </frame>
  
  </xsl:template>
  
  <xsl:template match="user-widget">

    <xsl:variable name="name">
      <xsl:choose>
        <xsl:when test="@label">
          <xsl:value-of select="@label"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="concat('u',generate-id())"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>

    <frame packside="left" pady="5">
      <user-widget procedure="{@procedure}" name="{$name}" variables="{@variables}" values="{@values}" parameters="{@parameters}" lang="{$lang}">
        <xsl:attribute name="text">
          <xsl:value-of select="strings[(@lang=$lang) or (not(@lang))]"/>
        </xsl:attribute>
      </user-widget>
    </frame>

  </xsl:template>
  
  <xsl:template match="scalar">

    <xsl:if test="@widget='slider'">

      <frame packside="left" padx="5" expand="true" fill="x">
        <label anchor="w" expand="true">
          <text>
            <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
          </text>
          <xsl:call-template name="helptext"/>
        </label>
        <scale anchor="e" variable="{@label}" state="{@state}" width="8" from="{@min}" to="{@max}" resolution="{@increment}">
          <value>
            <xsl:value-of select="value"/>
          </value>
          <command>runScilab</command>
        </scale>
      </frame>

    </xsl:if>

    <xsl:if test="@widget='entry'">

      <xsl:variable name="state">
        <xsl:choose>
          <xsl:when test="@state='disabled'">disabled</xsl:when>
          <xsl:when test="ancestor::database/@editable='no'">disabled</xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="@state"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:variable>

      <frame packside="left" padx="5" expand="true" fill="x">
        <label anchor="w" expand="true">
          <text>
            <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
          </text>
          <xsl:call-template name="helptext"/>
        </label>
        <entry anchor="e" variable="{@label}" state="{$state}" width="7">
          <value>
            <xsl:choose>
              <xsl:when test="string-length(normalize-space(value))&gt;0">
                <xsl:value-of select="value"/>
              </xsl:when>
              <xsl:otherwise>
                <xsl:value-of select="'0'"/>
              </xsl:otherwise>
            </xsl:choose>
          </value>
          <command>runScilab {}</command>
        </entry>
      </frame>

    </xsl:if>

    <xsl:if test="@widget='animate'">

      <frame packside="left" padx="5" expand="true" fill="x">
        <label anchor="w" expand="true">
          <text>
            <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
          </text>
          <xsl:call-template name="helptext"/>
        </label>
        <frame packside="left" padx="5" relief="groove">
          <scale anchor="e" variable="{@label}" state="{@state}" width="8" from="{@min}" to="{@max}" resolution="{@increment}">
            <value>
              <xsl:value-of select="value"/>
            </value>
            <command>runScilab</command>
          </scale>
          <jog variable="{@label}" from="{@min}" to="{@max}" resolution="{@increment}" period="{@period}">
            <command>runScilab {}</command>
          </jog>
        </frame>
      </frame>

    </xsl:if>

  </xsl:template>
  
  <xsl:template name="table-buttonbox">

    <buttonbox spacing="0">
      <xsl:if test="@save='yes'">
        <button image="saveImage">
          <command>
            <saveTable name="{@label}" rows="{@rows}" cols="{@cols}"/>
          </command>
          <helptext>
            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@savearray"/>
          </helptext>
        </button>
      </xsl:if>
      <xsl:if test="@load='yes'">
        <button image="loadImage">
          <command>
            <loadTable name="{@label}" rows="{@rows}" cols="{@cols}"/>
          </command>
          <helptext>
            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@loadarray"/>
          </helptext>
        </button>
      </xsl:if>
      <xsl:if test="@clear='yes'">
        <button image="deleteImage">
          <command>
            <eraseTable name="{@label}" rows="{@rows}" cols="{@cols}"/>
          </command>
          <helptext>
            <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@cleararray"/>
          </helptext>
        </button>
      </xsl:if>
    </buttonbox>

  </xsl:template>
  
  <xsl:template match="matrix">

    <frame packside="top" padx="5" pady="5" expand="true" fill="y">

      <frame packside="left" padx="5">
        <label anchor="c" expand="true" font="boldFont">
          <text>
            <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
          </text>
          <xsl:call-template name="helptext"/>
        </label>
        <xsl:if test="(@save='yes' or @load='yes' or @clear='yes') and not (col/name or row/name)">
          <xsl:call-template name="table-buttonbox"/>
        </xsl:if>
      </frame>

      <table name="{@label}" rows="{@rows}" cols="{@cols}" colheaders="{count(col[name])}" rowheaders="{count(row[name])}" state="{@state}">
        <xsl:if test="(@save='yes' or @load='yes' or @clear='yes') and (col/name or row/name)">
          <xsl:call-template name="table-buttonbox"/>
        </xsl:if>
      </table>

    </frame>

    <command>
      <xsl:call-template name="setmatrix"/>
      <xsl:choose>
        <xsl:when test="row/name">
          <setmatrix name="{@label}">
            <col from="1" number="0">
              <xsl:for-each select="row">
                <value>
                  <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
                </value>
              </xsl:for-each>
            </col>
          </setmatrix>
        </xsl:when>
        <xsl:when test="col/name">
          <setmatrix name="{@label}">
            <row from="1" number="0">
              <xsl:for-each select="col">
                <value>
                  <xsl:value-of select="normalize-space(name[(@lang=$lang) or (not(@lang))])"/>
                </value>
              </xsl:for-each>
            </row>
          </setmatrix>
        </xsl:when>
      </xsl:choose>
    </command>

  </xsl:template>
  
  <xsl:template name="setmatrix">
    <xsl:param name="label" select="@label"/>

    <setmatrix name="{$label}" mode="init">
      <xsl:for-each select="row|col">
        <xsl:element name="{name()}">
          <xsl:attribute name="number">
            <xsl:value-of select="position()"/>
          </xsl:attribute>
          <xsl:attribute name="from">1</xsl:attribute>
          <xsl:for-each select="str:split(translate(value,' ',','),',')">
            <value>
              <xsl:value-of select="."/>
            </value>
          </xsl:for-each>
        </xsl:element>
      </xsl:for-each>
    </setmatrix>

  </xsl:template>
  
  
  <xsl:template match="save" mode="menu">
    <item type="command">
      <text>
        <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@saveresults"/>
      </text>
      <description>
        <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@saveresultsdesc"/>
      </description>
      <command>
        <raise name="{translate(generate-id(),'.','_')}"/>
      </command>
    </item>
  </xsl:template>
  
  <xsl:template match="save" mode="pagemanager">

    <page name="{translate(generate-id(),'.','_')}" 
          text="{exslt:node-set($strings)/strings[@lang=$lang]/@saveresults}">

      <xsl:if test="//parameters/@widget='plain'">
        <frame packside="top" anchor="n" pady="5">
          <label font="boldFont">
            <text>
              <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@saveresults"/>
            </text>
          </label>
        </frame>
      </xsl:if>

      <xsl:apply-templates/>

      <button>
        <text>
          <xsl:value-of select="exslt:node-set($strings)/strings[@lang=$lang]/@save"/>
        </text>
        <command>ScilabEval exec(save_stuff,-1)</command>
      </button>

    </page>

  </xsl:template>
  
  <xsl:template match="file">

    <frame packside="left" padx="5" expand="true" fill="x">
      <label anchor="w" expand="true">
        <text>
          <xsl:value-of select="translate(@labels,' ','&#10;')"/>
        </text>
      </label>
      <label anchor="w" expand="true">
        <text>(Format : <xsl:value-of select="@format"/>)</text>
      </label>
      <entry variable="file{position()}" state="normal" width="8">
        <value>
          <xsl:value-of select="@href"/>
        </value>
      </entry>
    </frame>

  </xsl:template>
  
  <xsl:template match="headers"/>
  
  <xsl:template match="notes"/>
  
  <xsl:template match="title"/>
  
  <xsl:template match="display"/>
  
  <xsl:template match="save"/>
  
  <xsl:template match="image"/>

</xsl:stylesheet>
